"""Feature testing."""
import creopyson
import pytest
from .fixtures import (
    mk_creoson_post_dict,
    mk_creoson_post_None,
    mk_creoson_post_list,
    mk_getactivefile,
)


def test_file_assemble(mk_creoson_post_dict):
    """Test assemble."""
    c = creopyson.Client()
    result = c.file_assemble(
        "file",
        dirname="dirname",
        generic="generic",
        into_asm="target",
        path="path",
        ref_model="model",
        transform={"fake transform": 122},
        constraints={"fake constraint": 122},
        package_assembly=True,
        walk_children=True,
        assemble_to_root=True,
        suppress=True,
    )
    assert isinstance(result, (dict))


def test_file_backup(mk_creoson_post_None, mk_getactivefile):
    """Test backup."""
    c = creopyson.Client()
    result = c.file_backup("target", file_="file")
    assert result is None
    result = c.file_backup("target")
    assert result is None


def test_file_close_window(mk_creoson_post_None, mk_getactivefile):
    """Test close_window."""
    c = creopyson.Client()
    result = c.file_close_window(file_="file")
    assert result is None
    result = c.file_close_window()
    assert result is None


def test_file_display(mk_creoson_post_None):
    """Test display."""
    c = creopyson.Client()
    result = c.file_display("file", activate=True)
    assert result is None


def test_file_delete_material(mk_creoson_post_None, mk_getactivefile):
    """Test delete_material."""
    c = creopyson.Client()
    result = c.file_delete_material("brass", file_="file")
    assert result is None
    result = c.file_delete_material("brass")
    assert result is None


def test_file_erase(mk_creoson_post_None, mk_getactivefile):
    """Test erase."""
    c = creopyson.Client()
    result = c.file_erase(file_="file", erase_children=True)
    assert result is None
    result = c.file_erase(file_=["file", "other file"])
    assert result is None
    result = c.file_erase()
    assert result is None


def test_file_erase_not_displayed(mk_creoson_post_None):
    """Test erase_not_displayed."""
    c = creopyson.Client()
    result = c.file_erase_not_displayed()
    assert result is None


def test_file_exists(mk_creoson_post_dict):
    """Test exists."""
    c = creopyson.Client()
    result = c.file_exists("file")
    assert result is True


def test_file_get_accuracy(mk_creoson_post_dict, mk_getactivefile):
    """Test get_accuracy."""
    c = creopyson.Client()
    result = c.file_get_accuracy()
    assert isinstance(result, (dict))
    result = c.file_get_accuracy(file_="file")
    assert isinstance(result, (dict))


def test_file_get_active(mk_creoson_post_dict):
    """Test get_active."""
    c = creopyson.Client()
    result = c.file_get_active()
    assert isinstance(result, (dict))


def test_file_get_cur_material(mk_creoson_post_dict, mk_getactivefile):
    """Test get_cur_material."""
    c = creopyson.Client()
    result = c.file_get_cur_material(file_="file")
    assert isinstance(result, (str))
    result = c.file_get_cur_material()
    assert isinstance(result, (str))


def test_file_get_cur_material_wildcard(mk_creoson_post_dict, mk_getactivefile):
    """Test get_cur_material_wildcard."""
    c = creopyson.Client()
    result = c.file_get_cur_material_wildcard(
        file_="file", include_non_matching_parts=True
    )
    assert isinstance(result, (list))
    result = c.file_get_cur_material_wildcard()
    assert isinstance(result, (list))


def test_file_get_fileinfo(mk_creoson_post_dict, mk_getactivefile):
    """Test get_fileinfo."""
    c = creopyson.Client()
    result = c.file_get_fileinfo(file_="file")
    assert isinstance(result, (dict))
    result = c.file_get_fileinfo()
    assert isinstance(result, (dict))


def test_file_get_length_units(mk_creoson_post_dict, mk_getactivefile):
    """Test get_length_units."""
    c = creopyson.Client()
    result = c.file_get_length_units(file_="file")
    assert isinstance(result, (str))
    result = c.file_get_length_units()
    assert isinstance(result, (str))


def test_file_get_mass_units(mk_creoson_post_dict, mk_getactivefile):
    """Test get_mass_units."""
    c = creopyson.Client()
    result = c.file_get_mass_units(file_="file")
    assert isinstance(result, (str))
    result = c.file_get_mass_units()
    assert isinstance(result, (str))


def test_file_get_transform(mk_creoson_post_dict):
    """Test get_transform."""
    c = creopyson.Client()
    result = c.file_get_transform(asm="asm", path="path", csys="csys")
    assert isinstance(result, (dict))


def test_file_has_instances(mk_creoson_post_dict, mk_getactivefile):
    """Test has_instances."""
    c = creopyson.Client()
    result = c.file_has_instances(file_="file")
    assert result is True
    result = c.file_has_instances()
    assert result is True


def test_file_is_active(mk_creoson_post_dict, mk_getactivefile):
    """Test is_active."""
    c = creopyson.Client()
    result = c.file_is_active("file")
    assert result is True


def test_file_list(mk_creoson_post_dict, mk_getactivefile):
    """Test list."""
    c = creopyson.Client()
    result = c.file_list(file_="file")
    assert isinstance(result, (list))
    result = c.file_list(file_=["file", "other file"])
    assert isinstance(result, (list))
    result = c.file_list()
    assert isinstance(result, (list))


def test_file_list_instances(mk_creoson_post_dict, mk_getactivefile):
    """Test list_instances."""
    c = creopyson.Client()
    result = c.file_list_instances(file_="file")
    assert isinstance(result, (dict))
    result = c.file_list_instances()
    assert isinstance(result, (dict))


def test_file_list_materials(mk_creoson_post_dict, mk_getactivefile):
    """Test list_materials."""
    c = creopyson.Client()
    result = c.file_list_materials(file_="file", material="wood*")
    assert isinstance(result, (list))
    result = c.file_list_materials()
    assert isinstance(result, (list))


def test_file_list_materials_wildcard(mk_creoson_post_dict, mk_getactivefile):
    """Test list_materials."""
    c = creopyson.Client()
    result = c.file_list_materials_wildcard(
        file_="file", material="wood*", include_non_matching_parts=True
    )
    assert isinstance(result, (list))
    result = c.file_list_materials_wildcard()
    assert isinstance(result, (list))


def test_file_list_simp_reps(mk_creoson_post_dict, mk_getactivefile):
    """Test list_simp_reps."""
    c = creopyson.Client()
    result = c.file_list_simp_reps(file_="file", rep="rep")
    assert isinstance(result, (dict))
    result = c.file_list_simp_reps()
    assert isinstance(result, (dict))


def test_load_material_file(mk_creoson_post_dict, mk_getactivefile):
    """Test load_material_file"""
    c = creopyson.Client()
    result = c.file_load_material_file("material", file_="file", dirname="dirname")
    assert isinstance(result, (list))
    result = c.file_load_material_file("material")
    assert isinstance(result, (list))


def test_file_massprops(mk_creoson_post_dict, mk_getactivefile):
    """Test massprops."""
    c = creopyson.Client()
    result = c.file_massprops(file_="file")
    assert isinstance(result, (dict))
    result = c.file_massprops()
    assert isinstance(result, (dict))


def test_file_open(mk_creoson_post_dict, mk_getactivefile):
    """Test open."""
    c = creopyson.Client()
    result = c.file_open(
        "file",
        dirname="dirname",
        generic="generic",
        display=True,
        activate=True,
        new_window=True,
        regen_force=True,
    )
    assert isinstance(result, (dict))
    result = c.file_open(["file", "other file"],)
    assert isinstance(result, (dict))


def test_file_open_errors(mk_creoson_post_dict, mk_getactivefile):
    """Test open_errors."""
    c = creopyson.Client()
    result = c.file_open_errors(file_="file")
    assert result is True
    result = c.file_open_errors()
    assert result is True


def test_file_postregen_relations_get(mk_creoson_post_dict, mk_getactivefile):
    """Test postregen_relations_get."""
    c = creopyson.Client()
    result = c.file_postregen_relations_get(file_="file")
    assert isinstance(result, (list))
    result = c.file_postregen_relations_get()
    assert isinstance(result, (list))


def test_file_postregen_relations_set(mk_creoson_post_None, mk_getactivefile):
    """Test postregen_relations_set."""
    c = creopyson.Client()
    result = c.file_postregen_relations_set(file_="file", relations=["relation"])
    assert result is None
    result = c.file_postregen_relations_set()
    assert result is None


def test_file_refresh(mk_creoson_post_None, mk_getactivefile):
    """Test refresh."""
    c = creopyson.Client()
    result = c.file_refresh(file_="file")
    assert result is None
    result = c.file_refresh()
    assert result is None


def test_file_regenerate(mk_creoson_post_None, mk_getactivefile):
    """Test regenerate."""
    c = creopyson.Client()
    result = c.file_regenerate(file_="file", display=True)
    assert result is None
    result = c.file_regenerate(file_=["file", "other file"])
    assert result is None
    result = c.file_regenerate()
    assert result is None


def test_file_relations_get(mk_creoson_post_dict, mk_getactivefile):
    """Test relations_get."""
    c = creopyson.Client()
    result = c.file_relations_get(file_="file")
    assert isinstance(result, (list))
    result = c.file_relations_get()
    assert isinstance(result, (list))


def test_file_relations_set(mk_creoson_post_None, mk_getactivefile):
    """Test relations_set."""
    c = creopyson.Client()
    result = c.file_relations_set(file_="file", relations=["relation"])
    assert result is None
    result = c.file_relations_set()
    assert result is None


def test_file_rename(mk_creoson_post_dict, mk_getactivefile):
    """Test rename."""
    c = creopyson.Client()
    result = c.file_rename("new_name", file_="file", onlysession=True)
    assert isinstance(result, (str))
    result = c.file_rename("new_name")
    assert isinstance(result, (str))


def test_file_repaint(mk_creoson_post_None, mk_getactivefile):
    """Test repaint."""
    c = creopyson.Client()
    result = c.file_repaint(file_="file")
    assert result is None
    result = c.file_repaint()
    assert result is None


def test_file_save(mk_creoson_post_None, mk_getactivefile):
    """Test save."""
    c = creopyson.Client()
    result = c.file_save(file_="file")
    assert result is None
    result = c.file_save(file_=["file", "other file"])
    assert result is None
    result = c.file_save()
    assert result is None


def test_set_cur_material(mk_creoson_post_dict, mk_getactivefile):
    """Test set_cur_material."""
    c = creopyson.Client()
    result = c.file_set_cur_material("wood", file_="*.prt")
    assert isinstance(result, (list))
    result = c.file_set_cur_material("wood")
    assert isinstance(result, (list))


def test_file_set_length_units(mk_creoson_post_None, mk_getactivefile):
    """Test set_length_units."""
    c = creopyson.Client()
    result = c.file_set_length_units("mm", file_="file", convert=True)
    assert result is None
    result = c.file_set_length_units("mm")
    assert result is None
    result = c.file_set_length_units("mm", file_=["file1", "file2"])
    assert result is None
    result = c.file_set_length_units("mm")
    assert result is None


def test_file_set_mass_units(mk_creoson_post_None, mk_getactivefile):
    """Test set_mass_units."""
    c = creopyson.Client()
    result = c.file_set_mass_units("g", file_="file", convert=True)
    assert result is None
    result = c.file_set_mass_units("g", file_=["file1", "file2"])
    assert result is None
    result = c.file_set_mass_units("g")
    assert result is None



"""File module."""


def assemble(
    client,
    file_,
    dirname=None,
    generic=None,
    into_asm=None,
    path=None,
    ref_model=None,
    transform=None,
    constraints=None,
    package_assembly=None,
    walk_children=None,
    assemble_to_root=None,
    suppress=None,
):
    """Assemble a component into an assembly.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str):
            File name component.
        dirname (str, optional):
            Diretory name. Defaults is Creo's current working directory.
        generic (str, optional):
            Generic model name (if file name represents an instance).
            Defaults is generic model name (if file name represents an
            instance).
        into_asm (str, optional):
            Target assembly. Defaults is currently active model.
        path (list:int, optional):
            Path to a component that the new part will be constrained to.
            Defaults to None.
        ref_model (str, optional):
            Reference model that the new part will be constrained to;
            only used if path is not given.  If there are multiple of this
            model in the assembly, the component will be assembled multiple
            times, once to each occurrence. Defaults to None.
        transform (obj:JLTransform, optional):
            Transform structure for the initial position and orientation of
            the new component; only used if there are no constraints, or for
            certain constraint types. Defaults to None.
        constraints (obj_array:JLConstraint, optional):
            Assembly constraints. Defaults to None.
        package_assembly (bool, optional):
            Whether to package the component to the assembly; only used if
            there are no constraints specified. Defaults is If there are no
            constraints, then the user will be prompted to constrain the
            component through the Creo user interface.
        walk_children (bool, optional):
            Whether to walk into subassemblies to find reference models to
            constrain to. Defaults to None.
        assemble_to_root (bool, optional):
            Whether to always assemble to the root assembly, or assemble to
            the subassembly containing the reference path/model.
            Defaults to None.
        suppress (bool, optional):
            Whether to suppress the components immediately after assembling
            them. Defaults to None.

    Returns:
        (dict):
            dirname (str):
                Directory name of component.
            files (list:str):
                File name of component.
            revision (int):
                Revision of file that was opened; if more than one
                file was opened, this field is not returned.
            featureid (int):
                Last Feature ID of component after assembly.

    """
    data = {"file": file_}
    if dirname is not None:
        data["dirname"] = dirname
    if generic is not None:
        data["generic"] = generic
    if into_asm is not None:
        data["into_asm"] = into_asm
    if path is not None:
        data["path"] = path
    if ref_model is not None:
        data["ref_model"] = ref_model
    if transform is not None:
        data["transform"] = transform
    if constraints is not None:
        data["constraints"] = constraints
    if package_assembly is not None:
        data["package_assembly"] = package_assembly
    if walk_children is not None:
        data["walk_children"] = walk_children
    if assemble_to_root is not None:
        data["assemble_to_root"] = assemble_to_root
    if suppress is not None:
        data["suppress"] = suppress
    return client._creoson_post("file", "assemble", data)


def backup(client, target_dir, file_=None):
    """Backup a model.

    Args:
        client (obj):
            creopyson Client.
        target_dir (str):
            Target directory name.
        `file_` (str, optional):
            File name. Defaults is currently active model.

    Returns:
        None

    """
    data = {"target_dir": target_dir}
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    return client._creoson_post("file", "backup", data)


def close_window(client, file_=None):
    """Close the window containing a model.

    Args:
        client (obj):
            creopyson object.
        `file_` (str, optional):
            File name. Defaults is currently active model.

    Returns:
        None

    """
    data = {}
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    return client._creoson_post("file", "close_window", data)


def delete_material(client, material, file_=None):
    """Delete a material from a part.

    Args:
        client (obj):
            creopyson object
        material (str):
            Material name.
        `file_` (str, optional):
            File name.
            (Wildcards allowed: True).
            Defaults is currently active model.

    """
    data = {
        "material": material,
    }
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    return client._creoson_post("file", "delete_material", data)


def display(client, file_, activate=None):
    """Display a model in a window.

    Args:
        client (obj):
            creopyson object.
        `file_` (str):
            File name
        activate (bool, optional):
            Activate the model after displaying. Defaults is True.

    Returns:
        None

    """
    data = {"file": file_, "activate": True}
    if activate is not None:
        data["activate"] = activate
    return client._creoson_post("file", "display", data)


def erase(client, file_=None, erase_children=None):
    """Erase one or more models from memory.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str|list:str, optional):
            File name or List of file names;
            (Wildcards allowed: True).
            if empty all models in memory are erased.
        erase_children (bool, optional):
            Erase children of the models too. Defaults is False.

    Returns:
        None

    """
    data = {}
    if file_:
        if isinstance(file_, (str)):
            data["file"] = file_
        elif isinstance(file_, (list)):
            data["files"] = file_
    if erase_children is not None:
        data["erase_children"] = erase_children
    return client._creoson_post("file", "erase", data)


def erase_not_displayed(client):
    """Erase all non-displayed models from memory.

    Args:
        client (obj): creopyson Client.

    Returns:
        None

    """
    return client._creoson_post("file", "erase_not_displayed")


def exists(client, file_):
    """Check whether a model exists in memory.

    Args:
        client (obj): creopyson Client.
        `file_` (str): File name.

    Returns:
        (bool): Whether the file is open in Creo.

    """
    data = {"file": file_}
    return client._creoson_post("file", "exists", data, "exists")


def get_accuracy(client, file_=None):
    """Get a solid's accuracy.

    If the model has no accuracy value, this function will return null.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str, optional):
            File name.
            Defaults is current active model

    Returns:
        (list):
            accuracy (float):
                Accuracy value.
            relative (bool):
                True = relative; False = absolute accuracy.

    """
    data = {}
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    return client._creoson_post("file", "get_accuracy", data)


def get_active(client):
    """Get the active model from Creo.

    Args:
        client (obj): creopyson Client.

    Returns:
        (dict):
            dirname (str): Directory name of current model.
            file (str): File name of current model.

    """
    return client._creoson_post("file", "get_active")


def get_cur_material(client, file_=None):
    """Get the current material for a part.

    Note: This is the same as 'get_cur_material_wildcard' but this function
    does not allow wildcards on the part name. They are separate functions
    because the return structures are different. This function is retained
    for backwards compatibility.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str, optional):
            Part name. Defaults to None is current active model.

    Returns:
        str:
            Current material for the part, may be null if there is no
            current material.

    """
    data = {}
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    return client._creoson_post("file", "get_cur_material", data, "material")


def get_cur_material_wildcard(client, file_=None, include_non_matching_parts=False):
    """Get the current material for a part or parts.

    Note: This is the same as 'get_cur_material' but this function allows
    wildcards on the part name. They are separate functions because the return
    structures are different.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str, optional):
            Part name. Defaults to None is current active model.
        include_non_matching_parts (bool, optionnal):
            Whether to include parts that match the part name pattern but don't
            have a current material.
            Defaults to False.

    Returns:
        list:
            A list of part and current-material pairs.
    """
    data = {}
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    if include_non_matching_parts is not None:
        data["include_non_matching_parts"] = True
    return client._creoson_post("file", "get_cur_material_wildcard", data, "materials")


def get_fileinfo(client, file_=None):
    """Open one or more files in memory or from the drive.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str, optional):
            File name. Defaults is currently active model.

    Returns:
        (dict):
            dirname (str):
                Directory name of the file.
            file (str):
                File name.
            revision (int):
                Revision number of file.
                (Not availlable if the file is in Windchill)

    """
    data = {}
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    return client._creoson_post("file", "get_fileinfo", data)


def get_length_units(client, file_=None):
    """Get the current length units for a model.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str, optional):
            File name. Defaults is currently active model.

    Returns:
        (str): Length units.

    """
    data = {}
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    return client._creoson_post("file", "get_length_units", data, "units")


def get_mass_units(client, file_=None):
    """Get the current mass units for a model.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str, optional):
            File name. Defaults is currently active model.

    Returns:
        (str): mass units.

    """
    data = {}
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    return client._creoson_post("file", "get_mass_units", data, "units")


def get_transform(client, asm=None, path=None, csys=None):
    """Get the 3D transform for a component in an assembly.

    Args:
        client (obj):
            creopyson Client
        asm (str, optional):
            Assembly name. Defaults is currently active model.
        path (list:int, optional):
            Path to a component in the assembly.
            Defaults is the transform is calculated for the assembly itself.
        csys (str, optional):
            Coordinate system on the component to calculate the transform for.
            Defaults is the component's default coordinate system.

    Returns:
        (obj:JLTransform): The 3D transform from the assembly to
        the component's coordinate system.

    """
    data = {}
    if asm:
        data["asm"] = asm
    if path:
        data["path"] = path
    if csys:
        data["csys"] = csys
    return client._creoson_post("file", "get_transform", data)


def has_instances(client, file_=None):
    """Check whether a model has a family table.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str, optional):
            File name. Defaults is currently active model.

    Returns:
        (bool): Whether the file has a family table.

    """
    data = {}
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    return client._creoson_post("file", "has_instances", data, "exists")


def is_active(client, file_):
    """Check whether a model is the active model.

    Args:
        client (obj): creopyson Client.
        `file_` (str): File name.

    Returns:
        (bool): Whether the file is the currently active model.

    """
    data = {"file": file_}
    return client._creoson_post("file", "is_active", data, "active")


def list_(client, file_=None):
    """Get a list of files in the current Creo session that match patterns.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str|list:str, optional):
            File name or List of file names;

    Returns:
        (list:str) List of files.

    """
    data = {}
    if file_:
        if isinstance(file_, (str)):
            data["file"] = file_
        elif isinstance(file_, (list)):
            data["files"] = file_
    return client._creoson_post("file", "list", data, "files")


def list_instances(client, file_=None):
    """List instances in a model's family table.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str, optional):
            File name. Defaults is currently active model.

    Returns:
        (dict):
            dirname (str): Directory name of the file.
            generic (str): Generic name.
            files (list:str): List of model names in the table.

    """
    data = {}
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    return client._creoson_post("file", "list_instances", data)


def list_materials(client, file_=None, material=None):
    """List materials on a part.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str, optional):
            File name. Defaults is currently active model.
        material (str, optional):
            Material name pattern.
            Wildcards allowed.
            Defaults to None is all materials.

    Returns:
        list: List of materials in the part.

    """
    data = {}
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    if material is not None:
        data["material"] = material
    return client._creoson_post("file", "list_materials", data, "materials")


def list_materials_wildcard(
    client, file_=None, material=None, include_non_matching_parts=False
):
    """List materials on a part or parts.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str, optional):
            File name. Defaults is currently active model.
        material (str, optional):
            Material name pattern.
            Wildcards allowed.
            Defaults to None is all materials.
        include_non_matching_parts (bool, optional):
            Whether to include parts that match the part name pattern but
            don't have any materials matching the material pattern.
            Defaults to False.

    Returns:
        list:
            A list of part and material pairs. If a part has more than one
            material, it will have multiple entries in this array.
    """
    data = {}
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    if material is not None:
        data["material"] = material
    if include_non_matching_parts is not None:
        data["include_non_matching_parts"] = include_non_matching_parts
    return client._creoson_post("file", "list_materials_wildcard", data, "materials")


def list_simp_reps(client, file_=None, rep=None):
    """List simplified reps in a model.

    Args:
        client (obj):
            creopyson Client
        `file_` (str, optional):
            File name. Defaults is currently active model.
        rep (str, optional):
            Simplified rep name pattern (wildcards_allowed: True).
            Defaults is all simplified reps.

    Returns:
        (dict):
            rep (str): Simplified rep name.
            reps (list:str): Simplified reps names.

    """
    data = {}
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    if rep is not None:
        data["rep"] = rep
    return client._creoson_post("file", "list_simp_reps", data)


def load_material_file(client, material, dirname=None, file_=None):
    """Load a new material file into a part or parts.

    Note: If 'material' has a file extension, it will be removed before
    the material is loaded.

    Args:
        client (obj):
            creopyson Client
        material (str):
            Material name
        dirname (str, optional):
            Directory name containing the material file.
            Default is Creo's 'pro_material_dir' config setting,
            or search path, or current working directory
        `file_` (str, optional):
            File name.
            Wildcards allowed.
            Defaults is currently active model.

    Returns:
        list:
            List of files impacted.

    """
    data = {"dirname": dirname, "material": material}
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    return client._creoson_post("file", "load_material_file", data, "files")


def massprops(client, file_=None):
    """Get mass property information about a model.

    Notes: PTC's description of coord_sys_inertia:
    "The inertia matrix with respect to coordinate frame:(element ij is
    the integral of x_i x_j over the object)".
    PTC's description of coord_sys_inertia_tensor:
    "The inertia tensor with respect to coordinate frame:
    CoordSysInertiaTensor =
    trace(CoordSysInertia) * identity - CoordSysInertia".

    Args:
        client (obj):
            creopyson Client.
        `file_` (str, optional):
            File name. Defaults is currently active model.

    Returns:
        (dict):
            volume (float):
                Model volume.
            mass (float):
                Model mass.
            density (float):
                Model density.
            surface_area (float):
                Model surface area.
            ctr_grav_inertia_tensor (object:JLInertia):
                Model's Inertia Tensor translated to center of gravity.
            coord_sys_inertia (object:JLInertia):
                Model's Inertia Matrix with respect to the coordinate frame.
            coord_sys_inertia_tensor (object:JLInertia):
                Model's Inertia Tensor with respect to the coordinate frame.
            ctr_grav (object:JLPoint):
                Model's center of gravity.

    """
    data = {}
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    return client._creoson_post("file", "massprops", data)


def open_(
    client,
    file_,
    dirname=None,
    generic=None,
    display=None,
    activate=None,
    new_window=None,
    regen_force=None,
):
    """Open one or more files in memory or from the drive.

    note: if you open more than one file, it will only put file in your session
    you won't be able to display more than one file at once.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str|list:str):
            File name or List of file names;
        dirname (str, optional):
            Directory name. Defaults is Creo's current working directory.
        generic (str, optional):
            Generic model name (if file name represents an instance).
            Defaults to None.
        display (bool, optional):
            Display the model after opening. Defaults is True.
        activate (bool, optional):
            Activate the model after opening. Defaults is True.
        new_window (bool, optional):
            Open model in a new window. Defaults is False.
        regen_force (bool, optional):
            Force regeneration after opening. Defaults is False.

    Returns:
        (dict):
            dirname (str):
                Directory name of opened file(s).
            files (list:str):
                File names that were opened.
            revision (int):
                Revision of file that was opened;
                if more than one file was opened, this field is not returned.

    """
    data = {
        "display": True,
        "activate": True,
    }
    if file_:
        if isinstance(file_, (str)):
            data["file"] = file_
        elif isinstance(file_, (list)):
            data["files"] = file_
    if dirname is not None:
        data["dirname"] = dirname
    if generic is not None:
        data["generic"] = generic
    if display is not None:
        data["display"] = display
    if activate is not None:
        data["activate"] = activate
    if new_window is not None:
        data["new_window"] = new_window
    if regen_force is not None:
        data["regen_force"] = regen_force
    return client._creoson_post("file", "open", data)
    # TODO


def open_errors(client, file_=None):
    """Check whether Creo errors have occurred opening a model.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str, optional):
            File name. Defaults is currently active model.

    Returns:
        (bool): Whether errors exist in Creo.

    """
    data = {}
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    return client._creoson_post("file", "open_errors", data, "errors")


def postregen_relations_get(client, file_=None):
    """Get post-regeneration relations for a model.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str, optional):
            File name. Defaults is currently active model.

    Returns:
        (list:str): Exported relations text, one entry per line.

    """
    data = {}
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    return client._creoson_post("file", "postregen_relations_get", data, "relations")


def postregen_relations_set(client, file_=None, relations=None):
    """Set post-regeneration relations for a model.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str, optional):
            File name. Defaults is currently active model.
        relations (list:str, optional):
            Relations text to import, one line per entry.
            Clear the relations if missing.

    Retunrs:
        None

    """
    data = {}
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    if relations is not None:
        data["relations"] = relations
    return client._creoson_post("file", "postregen_relations_set", data)


def refresh(client, file_=None):
    """Refresh the window containing a model.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str, optional):
            File name. Defaults is currently active model.

    Returns:
        None

    """
    data = {}
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    return client._creoson_post("file", "refresh", data)


def regenerate(client, file_=None, display=None):
    """Regenerate one or more models.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str|list:str, optional):
            File name or List of file names;
            Defaults is currently active model
        display (bool, optional):
            Display the model before regenerating. Defaults is False.

    Returns:
        None

    """
    data = {}
    if file_ is not None:
        if isinstance(file_, (str)):
            data["file"] = file_
        elif isinstance(file_, (list)):
            data["files"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    if display is not None:
        data["display"] = display
    return client._creoson_post("file", "regenerate", data)


def relations_get(client, file_=None):
    """Get relations for a model.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str, optional):
            File name. Defaults is currently active model.

    Returns:
        (list:srt): Exported relations text, one entry per line.

    """
    data = {}
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    return client._creoson_post("file", "relations_get", data, "relations")


def relations_set(client, file_=None, relations=None):
    """Set relations for a model.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str, optional):
            File name. Defaults is currently active model.
        relations (list:str, optional):
            Relations text to import, one line per entry.
            Clear the relations if missing.

    Returns:
        None

    """
    data = {}
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    if relations is not None:
        data["relations"] = relations
    return client._creoson_post("file", "relations_set", data)


def rename(client, new_name, file_=None, onlysession=None):
    """Rename a model.

    Args:
        client (obj):
            creopyson Client.
        new_name (str):
            New file name.
        `file_` (str, optional):
            File name. Defaults is currently active model.
        onlysession (bool, optional):
            Modify only in memory, not on disk. Defaults is False.

    Returns:
        (str): The new model name.

    """
    data = {"new_name": new_name, "onlysession": False}
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    if onlysession is not None:
        data["onlysession"] = onlysession
    return client._creoson_post("file", "rename", data, "file")


def repaint(client, file_=None):
    """Repaint the window containing a model.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str, optional):
            File name. Defaults is currently active model.

    Returns:
        None

    """
    data = {}
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    return client._creoson_post("file", "repaint", data)


def save(client, file_=None):
    """Save one or more models.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str|list:str, optional):
            File name or List of file names;
            Defaults is currently active model.

    Returns:
        None

    """
    data = {}
    if file_ is not None:
        if isinstance(file_, (str)):
            data["file"] = file_
        elif isinstance(file_, (list)):
            data["files"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    return client._creoson_post("file", "save", data)


def set_cur_material(client, material, file_=None):
    """Set the current material for a part or parts.

    Args:
        client (obj):
            creopyson Client.
        material (str):
            Material name.
        `file_` (str, optional):
            Part name. Wildcard allowed.
            Defaults is currently active model.

    Returns:
        list:
            list of impacted files.

    """
    data = {"material": material}
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    return client._creoson_post("file", "set_cur_material", data, "files")


def set_length_units(client, units, file_=None, convert=None):
    """Set the current length units for a model.

    This will search the model's available Unit Systems for the first one
    which contains the given length unit.

    Args:
        client (obj):
            creopyson Client.
        units (str):
            New length units.
        `file_` (str|list:str, optional):
            File name or List of file names;
            Defaults is currently active model.
        convert (bool, optional):
            Whether to convert the model's length values to the
            new units (True) or leave them the same value (False).
            Defaults is True.

    Returns:
        None

    """
    data = {"units": units}
    if file_ is not None:
        if isinstance(file_, (str)):
            data["file"] = file_
        elif isinstance(file_, (list)):
            data["files"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    if convert is not None:
        data["convert"] = convert
    return client._creoson_post("file", "set_length_units", data)


def set_mass_units(client, units, file_=None, convert=None):
    """Set the mass units for a model.

    This will search the model's available Unit Systems for the first one
    which contains the given mass unit.

    Args:
        client (obj):
            creopyson Client.
        units (str):
            New mass units.
        `file_` (str|list:str, optional):
            File name or List of file names;
            Defaults is currently active model.
        convert (bool, optional):
            Whether to convert the model's mass values to the
            new units (True) or leave them the same value (False).
            Defaults is True.

    Returns:
        None

    """
    data = {"units": units}
    if file_ is not None:
        if isinstance(file_, (str)):
            data["file"] = file_
        elif isinstance(file_, (list)):
            data["files"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    if convert is not None:
        data["convert"] = convert
    return client._creoson_post("file", "set_mass_units", data)



"""Geometry testing."""
import creopyson
from .fixtures import mk_creoson_post_dict, mk_creoson_post_None, mk_getactivefile


def test_geometry_bound_box(mk_creoson_post_dict, mk_getactivefile):
    """Test bound_box."""
    c = creopyson.Client()
    result = c.geometry_bound_box(file_="file")
    assert isinstance(result, (dict))
    result = c.geometry_bound_box()
    assert isinstance(result, (dict))


def test_geometry_get_edges(mk_creoson_post_dict, mk_getactivefile):
    """Test get_edges."""
    c = creopyson.Client()
    result = c.geometry_get_edges(["12", "34"], file_="file")
    assert isinstance(result, (list))
    result = c.geometry_get_edges(["12", "34"])
    assert isinstance(result, (list))


def test_geometry_get_surfaces(mk_creoson_post_dict, mk_getactivefile):
    """Test get_surfaces."""
    c = creopyson.Client()
    result = c.geometry_get_surfaces(file_="file")
    assert isinstance(result, (list))
    result = c.geometry_get_surfaces()
    assert isinstance(result, (list))



"""Parameters testing."""
import creopyson
from .fixtures import mk_creoson_post_dict, mk_creoson_post_None, mk_getactivefile


def test_parameter_copy(mk_creoson_post_None, mk_getactivefile):
    """Test copy."""
    c = creopyson.Client()
    result = c.parameter_copy(
        "name", "to_name", file_="file", to_file="target file", designate=True
    )
    assert result is None
    result = c.parameter_copy("name", "to_name")
    assert result is None


def test_parameter_delete(mk_creoson_post_None, mk_getactivefile):
    """Test delete."""
    c = creopyson.Client()
    result = c.parameter_delete("name", file_="file")
    assert result is None
    result = c.parameter_delete("name")
    assert result is None


def test_parameter_exists(mk_creoson_post_dict, mk_getactivefile):
    """Test exists."""
    c = creopyson.Client()
    result = c.parameter_exists(name="name", file_="file")
    assert result is True
    result = c.parameter_exists(name=["name", "other name"])
    assert result is True


def test_parameter_list(mk_creoson_post_dict, mk_getactivefile):
    """Test list."""
    c = creopyson.Client()
    result = c.parameter_list(name="name", file_="file", encoded=True, value="value")
    assert isinstance(result, (list))
    result = c.parameter_list(name=["name", "other name"])
    assert isinstance(result, (list))


def test_parameter_set(mk_creoson_post_None, mk_getactivefile):
    """Test set."""
    c = creopyson.Client()
    result = c.parameter_set(
        "name",
        value=12,
        file_="file",
        type_="INTEGER",
        encoded=True,
        designate=True,
        description="This is a description",
        no_create=True,
    )
    assert result is None
    result = c.parameter_set("name")
    assert result is None


def test_parameter_set_designated(mk_creoson_post_None, mk_getactivefile):
    """Test set_designated."""
    c = creopyson.Client()
    result = c.parameter_set_designated("name", True, file_="file")
    assert result is None
    result = c.parameter_set_designated("name", True)
    assert result is None



"""Connection module."""
import requests
import json
import logging
from pathlib import Path
from .exceptions import MissingKey, ErrorJsonDecode

lg = logging.getLogger(__name__)


class Client(object):
    """Creates Client object."""

    def __init__(self, ip_adress="localhost", port=9056):
        """Create Client objet. Define server and sessionID vars."""
        self.server = "http://{}:{}/creoson".format(ip_adress, port)
        self.sessionId = ""

    def connect(self):
        """Connect to CREOSON.

        Define 'sessionId'.
        Exit if server not found.
        """
        self.sessionId = self._creoson_post("connection", "connect")

    def _creoson_post(self, command, function, data=None, key_data=None):
        """Send a POST request to creoson server and return waited data.

        Args:
            command (str): Command param for creoson.
            function (str): Function param for creoson.
            data (dict, optionnal): data params for creson request.
            key_data (str, optionnal): param name waited in result.

        Raises:
            RuntimeError: error message from creoson.
            ConnectionError: creoson not reachable.
            MissingKey: Missing arg in creoson return.

        Returns:
            (depends request): creoson return.

        """
        request = {
            "sessionId": self.sessionId,
            "command": command,
            "function": function,
            "data": data,
        }
        lg.debug("request: %s", str(request))
        try:
            r = requests.post(self.server, data=json.dumps(request))
        except requests.exceptions.RequestException as e:
            raise ConnectionError(e)

        if r.status_code != 200:
            raise ConnectionError("Status code : {}".format(r.status_code))

        try:
            json_result = r.json()
            lg.debug("response: %s", str(json_result))
        except TypeError:
            raise ErrorJsonDecode("Cannot decode JSON, creoson result invalid.")

        if "status" not in json_result.keys():
            raise MissingKey("Missing `status` in creoson result.")

        if "error" not in json_result["status"].keys():
            raise MissingKey("Missing `error` in status' creoson's result.")

        status = json_result["status"]["error"]
        if status:
            error_msg = json_result["status"]["message"]
            raise RuntimeError(error_msg)

        if request["command"] == "connection" and request["function"] == "connect":
            if "sessionId" not in json_result.keys():
                raise MissingKey("Missing `sessionId` in creoson result.")
            else:
                return json_result["sessionId"]

        if key_data is not None:
            if "data" not in json_result.keys():
                raise MissingKey("Missing `data` in creoson return")
            if key_data not in json_result["data"].keys():
                raise MissingKey("Missing `{}` in creoson result".format(key_data))
            return json_result["data"][key_data]

        return json_result.get("data", None)

    def disconnect(self):
        """Disconnect from CREOSON.

        Empty sessionId.
        """
        self._creoson_post("connection", "disconnect")
        self.sessionId = ""

    def is_creo_running(self):
        """Check whether Creo is running.

        This function tests whether the current connection is still active;
        if there is no active connection, then it tries to make a new
        connection to Creo and returns whether the connection succeeds.
        The sessionId is optional, and ignored.

        Raises:
            Warning: error message from creoson.

        Returns:
            (boolean): True if Creo is running, False instead.

        """
        # return self._creoson_post("connection", "is_creo_running")["running"]
        return self._creoson_post("connection", "is_creo_running", key_data="running")

    def kill_creo(self):
        """Kill primary Creo processes.

        This will kill the 'xtop.exe' and 'nmsd.exe' processes by name.
        The sessionId is optional, and ignored.

        Raises:
            Warning: error message from creoson.

        Returns:
            None

        """
        return self._creoson_post("connection", "kill_creo")

    def start_creo(self, path, retries=0, use_desktop=False):
        """Execute an external .bat file to start Creo.

        Then attempts to connect to Creo.

        The .bat file is restricted to a specific name to make the
        function more secure. (nitro_proe_remote.bat)
        Set retries to 0 to NOT attempt to connect to Creo.
        The server will pause for 3 seconds before attempting a
        connection, and will pause for 10 seconds between
        connection retries.
        If Creo pops up a message after startup, this function may
        cause Creo to crash unless retries is set to 0.
        If use_desktop is set, make sure that your
        nitro_proe_remote.bat file contains a cd command to change
        to the directory where you want Creo to start!

        Args:
            path (string):
                path to the .bat file (must be full path)
                will be split in 'start_command' and 'start_dir'
            retries (int):
                Number of retries to make when connecting
                (default 0)
            use_desktop (boolean):
                Whether to use the desktop to start creo rather than
                the java runtime. Should only be used if the runtime
                method doesn't work.
                Default is False.

        Raises:
            Warning: error message from creoson.

        Returns:
            None

        """
        path_obj = Path(path)
        start_command = path_obj.name
        start_dir = str(path_obj.parents[0])
        data = {
            "start_dir": start_dir,
            "start_command": start_command,
            "retries": retries,
            "use_desktop": use_desktop,
        }
        return self._creoson_post("connection", "start_creo", data)

    def stop_creo(self):
        """Disconnect current session from Creo and cause Creo to exit.

        NOTE that this will cause Creo to exit cleanly.
        If there is no current connection to Creo, this function
        will do nothing.

        Raises:
            Warning: error message from creoson.

        Returns:
            None

        """
        return self._creoson_post("connection", "stop_creo")



# -*- coding: utf-8 -*-

"""Top-level package for Creopyson."""

__author__ = """Benjamin C."""
__email__ = "zepman@gmail.com"
__version__ = "0.7.6"

from creopyson.connection import Client
from creopyson.objects import jlpoint


"""Add API methods to CLient."""

# Bom
from creopyson.bom import get_paths as bom_get_paths

Client.bom_get_paths = bom_get_paths


# Creo
from creopyson.creo import cd as creo_cd
from creopyson.creo import delete_files as creo_delete_files
from creopyson.creo import get_config as creo_get_config
from creopyson.creo import get_std_color as creo_get_std_color
from creopyson.creo import list_dirs as creo_list_dirs
from creopyson.creo import list_files as creo_list_files
from creopyson.creo import mkdir as creo_mkdir
from creopyson.creo import pwd as creo_pwd
from creopyson.creo import rmdir as creo_rmdir
from creopyson.creo import set_config as creo_set_config
from creopyson.creo import set_creo_version as creo_set_creo_version
from creopyson.creo import set_std_color as creo_set_std_color

Client.creo_cd = creo_cd
Client.creo_delete_files = creo_delete_files
Client.creo_get_config = creo_get_config
Client.creo_get_std_color = creo_get_std_color
Client.creo_list_dirs = creo_list_dirs
Client.creo_list_files = creo_list_files
Client.creo_mkdir = creo_mkdir
Client.creo_pwd = creo_pwd
Client.creo_rmdir = creo_rmdir
Client.creo_set_config = creo_set_config
Client.creo_set_creo_version = creo_set_creo_version
Client.creo_set_std_color = creo_set_std_color


# Dimension
from creopyson.dimension import copy as dimension_copy
from creopyson.dimension import list_detail as dimension_list_detail
from creopyson.dimension import list_ as dimension_list
from creopyson.dimension import set_ as dimension_set
from creopyson.dimension import set_text as dimension_set_text
from creopyson.dimension import show as dimension_show
from creopyson.dimension import user_select as dimension_user_select

Client.dimension_copy = dimension_copy
Client.dimension_list_detail = dimension_list_detail
Client.dimension_list = dimension_list
Client.dimension_set = dimension_set
Client.dimension_set_text = dimension_set_text
Client.dimension_show = dimension_show
Client.dimension_user_select = dimension_user_select


# Drawing
from creopyson.drawing import add_model as drawing_add_model
from creopyson.drawing import add_sheet as drawing_add_sheet
from creopyson.drawing import create_gen_view as drawing_create_gen_view
from creopyson.drawing import create as drawing_create
from creopyson.drawing import create_proj_view as drawing_create_proj_view
from creopyson.drawing import create_symbol as drawing_create_symbol
from creopyson.drawing import delete_models as drawing_delete_models
from creopyson.drawing import delete_sheet as drawing_delete_sheet
from creopyson.drawing import delete_symbol_def as drawing_delete_symbol_def
from creopyson.drawing import delete_symbol_inst as drawing_delete_symbol_inst
from creopyson.drawing import delete_view as drawing_delete_view
from creopyson.drawing import get_cur_model as drawing_get_cur_model
from creopyson.drawing import get_cur_sheet as drawing_get_cur_sheet
from creopyson.drawing import get_num_sheets as drawing_get_num_sheets
from creopyson.drawing import get_sheet_format as drawing_get_sheet_format
from creopyson.drawing import get_sheet_scale as drawing_get_sheet_scale
from creopyson.drawing import get_sheet_size as drawing_get_sheet_size
from creopyson.drawing import get_view_loc as drawing_get_view_loc
from creopyson.drawing import get_view_scale as drawing_get_view_scale
from creopyson.drawing import get_view_sheet as drawing_get_view_sheet
from creopyson.drawing import is_symbol_def_loaded as drawing_is_symbol_def_loaded
from creopyson.drawing import list_models as drawing_list_models
from creopyson.drawing import list_symbols as drawing_list_symbols
from creopyson.drawing import list_view_details as drawing_list_view_details
from creopyson.drawing import list_views as drawing_list_views
from creopyson.drawing import load_symbol_def as drawing_load_symbol_def
from creopyson.drawing import regenerate as drawing_regenerate
from creopyson.drawing import regenerate_sheet as drawing_regenerate_sheet
from creopyson.drawing import rename_view as drawing_rename_view
from creopyson.drawing import scale_sheet as drawing_scale_sheet
from creopyson.drawing import scale_view as drawing_scale_view
from creopyson.drawing import select_sheet as drawing_select_sheet
from creopyson.drawing import set_cur_model as drawing_set_cur_model
from creopyson.drawing import set_sheet_format as drawing_set_sheet_format
from creopyson.drawing import set_view_loc as drawing_set_view_loc
from creopyson.drawing import view_bound_box as drawing_view_bound_box

Client.drawing_add_model = drawing_add_model
Client.drawing_add_sheet = drawing_add_sheet
Client.drawing_create_gen_view = drawing_create_gen_view
Client.drawing_create = drawing_create
Client.drawing_create_proj_view = drawing_create_proj_view
Client.drawing_create_symbol = drawing_create_symbol
Client.drawing_delete_models = drawing_delete_models
Client.drawing_delete_sheet = drawing_delete_sheet
Client.drawing_delete_symbol_def = drawing_delete_symbol_def
Client.drawing_delete_symbol_inst = drawing_delete_symbol_inst
Client.drawing_delete_view = drawing_delete_view
Client.drawing_get_cur_model = drawing_get_cur_model
Client.drawing_get_cur_sheet = drawing_get_cur_sheet
Client.drawing_get_num_sheets = drawing_get_num_sheets
Client.drawing_get_sheet_format = drawing_get_sheet_format
Client.drawing_get_sheet_scale = drawing_get_sheet_scale
Client.drawing_get_sheet_size = drawing_get_sheet_size
Client.drawing_get_view_loc = drawing_get_view_loc
Client.drawing_get_view_scale = drawing_get_view_scale
Client.drawing_get_view_sheet = drawing_get_view_sheet
Client.drawing_is_symbol_def_loaded = drawing_is_symbol_def_loaded
Client.drawing_list_models = drawing_list_models
Client.drawing_list_symbols = drawing_list_symbols
Client.drawing_list_view_details = drawing_list_view_details
Client.drawing_list_views = drawing_list_views
Client.drawing_load_symbol_def = drawing_load_symbol_def
Client.drawing_regenerate = drawing_regenerate
Client.drawing_regenerate_sheet = drawing_regenerate_sheet
Client.drawing_rename_view = drawing_rename_view
Client.drawing_scale_sheet = drawing_scale_sheet
Client.drawing_scale_view = drawing_scale_view
Client.drawing_select_sheet = drawing_select_sheet
Client.drawing_set_cur_model = drawing_set_cur_model
Client.drawing_set_sheet_format = drawing_set_sheet_format
Client.drawing_set_view_loc = drawing_set_view_loc
Client.drawing_view_bound_box = drawing_view_bound_box


# Familytable
from creopyson.familytable import add_inst as familytable_add_inst
from creopyson.familytable import create_inst as familytable_create_inst
from creopyson.familytable import delete_inst as familytable_delete_inst
from creopyson.familytable import delete as familytable_delete
from creopyson.familytable import exists as familytable_exists
from creopyson.familytable import get_cell as familytable_get_cell
from creopyson.familytable import get_header as familytable_get_header
from creopyson.familytable import get_parents as familytable_get_parents
from creopyson.familytable import get_row as familytable_get_row
from creopyson.familytable import list_ as familytable_list
from creopyson.familytable import list_tree as familytable_list_tree
from creopyson.familytable import replace as familytable_replace
from creopyson.familytable import set_cell as familytable_set_cell

Client.familytable_add_inst = familytable_add_inst
Client.familytable_create_inst = familytable_create_inst
Client.familytable_delete_inst = familytable_delete_inst
Client.familytable_delete = familytable_delete
Client.familytable_exists = familytable_exists
Client.familytable_get_cell = familytable_get_cell
Client.familytable_get_header = familytable_get_header
Client.familytable_get_parents = familytable_get_parents
Client.familytable_get_row = familytable_get_row
Client.familytable_list = familytable_list
Client.familytable_list_tree = familytable_list_tree
Client.familytable_replace = familytable_replace
Client.familytable_set_cell = familytable_set_cell


# Feature
from creopyson.feature import delete as feature_delete
from creopyson.feature import delete_param as feature_delete_param
from creopyson.feature import list_ as feature_list
from creopyson.feature import list_params as feature_list_params
from creopyson.feature import list_group_features as feature_list_group_features
from creopyson.feature import list_pattern_features as feature_list_pattern_features
from creopyson.feature import list_selected as feature_list_selected
from creopyson.feature import param_exists as feature_param_exists
from creopyson.feature import rename as feature_rename
from creopyson.feature import resume as feature_resume
from creopyson.feature import set_param as feature_set_param
from creopyson.feature import suppress as feature_suppress
from creopyson.feature import user_select_csys as feature_user_select_csys

Client.feature_delete = feature_delete
Client.feature_delete_param = feature_delete_param
Client.feature_list = feature_list
Client.feature_list_params = feature_list_params
Client.feature_list_group_features = feature_list_group_features
Client.feature_list_pattern_features = feature_list_pattern_features
Client.feature_list_selected = feature_list_selected
Client.feature_param_exists = feature_param_exists
Client.feature_rename = feature_rename
Client.feature_resume = feature_resume
Client.feature_set_param = feature_set_param
Client.feature_suppress = feature_suppress
Client.feature_user_select_csys = feature_user_select_csys


# File
from creopyson.file import assemble as file_assemble
from creopyson.file import backup as file_backup
from creopyson.file import close_window as file_close_window
from creopyson.file import display as file_display
from creopyson.file import delete_material as file_delete_material
from creopyson.file import erase as file_erase
from creopyson.file import erase_not_displayed as file_erase_not_displayed
from creopyson.file import exists as file_exists
from creopyson.file import get_active as file_get_active
from creopyson.file import get_accuracy as file_get_accuracy
from creopyson.file import get_cur_material as file_get_cur_material
from creopyson.file import get_cur_material_wildcard as file_get_cur_material_wildcard
from creopyson.file import get_fileinfo as file_get_fileinfo
from creopyson.file import get_length_units as file_get_length_units
from creopyson.file import get_mass_units as file_get_mass_units
from creopyson.file import get_transform as file_get_transform
from creopyson.file import has_instances as file_has_instances
from creopyson.file import is_active as file_is_active
from creopyson.file import list_instances as file_list_instances
from creopyson.file import list_materials as file_list_materials
from creopyson.file import list_materials_wildcard as file_list_materials_wildcard
from creopyson.file import list_ as file_list
from creopyson.file import list_simp_reps as file_list_simp_reps
from creopyson.file import load_material_file as file_load_material_file
from creopyson.file import massprops as file_massprops
from creopyson.file import open_errors as file_open_errors
from creopyson.file import open_ as file_open
from creopyson.file import postregen_relations_get as file_postregen_relations_get
from creopyson.file import postregen_relations_set as file_postregen_relations_set
from creopyson.file import refresh as file_refresh
from creopyson.file import regenerate as file_regenerate
from creopyson.file import relations_get as file_relations_get
from creopyson.file import relations_set as file_relations_set
from creopyson.file import rename as file_rename
from creopyson.file import repaint as file_repaint
from creopyson.file import save as file_save
from creopyson.file import set_cur_material as file_set_cur_material
from creopyson.file import set_length_units as file_set_length_units
from creopyson.file import set_mass_units as file_set_mass_units

Client.file_assemble = file_assemble
Client.file_backup = file_backup
Client.file_close_window = file_close_window
Client.file_display = file_display
Client.file_delete_material = file_delete_material
Client.file_erase = file_erase
Client.file_erase_not_displayed = file_erase_not_displayed
Client.file_exists = file_exists
Client.file_get_active = file_get_active
Client.file_get_accuracy = file_get_accuracy
Client.file_get_cur_material = file_get_cur_material
Client.file_get_cur_material_wildcard = file_get_cur_material_wildcard
Client.file_get_fileinfo = file_get_fileinfo
Client.file_get_length_units = file_get_length_units
Client.file_get_mass_units = file_get_mass_units
Client.file_get_transform = file_get_transform
Client.file_has_instances = file_has_instances
Client.file_is_active = file_is_active
Client.file_list_instances = file_list_instances
Client.file_list_materials = file_list_materials
Client.file_list_materials_wildcard = file_list_materials_wildcard
Client.file_list = file_list
Client.file_list_simp_reps = file_list_simp_reps
Client.file_load_material_file = file_load_material_file
Client.file_massprops = file_massprops
Client.file_open_errors = file_open_errors
Client.file_open = file_open
Client.file_postregen_relations_get = file_postregen_relations_get
Client.file_postregen_relations_set = file_postregen_relations_set
Client.file_refresh = file_refresh
Client.file_regenerate = file_regenerate
Client.file_relations_get = file_relations_get
Client.file_relations_set = file_relations_set
Client.file_rename = file_rename
Client.file_repaint = file_repaint
Client.file_save = file_save
Client.file_set_cur_material = file_set_cur_material
Client.file_set_length_units = file_set_length_units
Client.file_set_mass_units = file_set_mass_units


# Geometry
from creopyson.geometry import bound_box as geometry_bound_box
from creopyson.geometry import get_edges as geometry_get_edges
from creopyson.geometry import get_surfaces as geometry_get_surfaces

Client.geometry_bound_box = geometry_bound_box
Client.geometry_get_edges = geometry_get_edges
Client.geometry_get_surfaces = geometry_get_surfaces


# Interface
from creopyson.interface import export_3dpdf as interface_export_3dpdf
from creopyson.interface import export_file as interface_export_file
from creopyson.interface import export_image as interface_export_image
from creopyson.interface import export_pdf as interface_export_pdf
from creopyson.interface import export_program as interface_export_program
from creopyson.interface import import_file as interface_import_file
from creopyson.interface import import_program as interface_import_program
from creopyson.interface import mapkey as interface_mapkey
from creopyson.interface import plot as interface_plot

Client.interface_export_3dpdf = interface_export_3dpdf
Client.interface_export_file = interface_export_file
Client.interface_export_image = interface_export_image
Client.interface_export_pdf = interface_export_pdf
Client.interface_export_program = interface_export_program
Client.interface_import_file = interface_import_file
Client.interface_import_program = interface_import_program
Client.interface_mapkey = interface_mapkey
Client.interface_plot = interface_plot


# Layer
from creopyson.layer import delete as layer_delete
from creopyson.layer import exists as layer_exists
from creopyson.layer import list_ as layer_list
from creopyson.layer import show as layer_show

Client.layer_delete = layer_delete
Client.layer_exists = layer_exists
Client.layer_list = layer_list
Client.layer_show = layer_show


# Note
from creopyson.note import copy as note_copy
from creopyson.note import delete as note_delete
from creopyson.note import exists as note_exists
from creopyson.note import get as note_get
from creopyson.note import list_ as note_list
from creopyson.note import set_ as note_set

Client.note_copy = note_copy
Client.note_delete = note_delete
Client.note_exists = note_exists
Client.note_get = note_get
Client.note_list = note_list
Client.note_set = note_set


# Parameter
from creopyson.parameter import copy as parameter_copy
from creopyson.parameter import delete as parameter_delete
from creopyson.parameter import exists as parameter_exists
from creopyson.parameter import list_ as parameter_list
from creopyson.parameter import set_ as parameter_set
from creopyson.parameter import set_designated as parameter_set_designated

Client.parameter_copy = parameter_copy
Client.parameter_delete = parameter_delete
Client.parameter_exists = parameter_exists
Client.parameter_list = parameter_list
Client.parameter_set = parameter_set
Client.parameter_set_designated = parameter_set_designated


# Server
from creopyson.server import pwd as server_pwd

Client.server_pwd = server_pwd


# View
from creopyson.view import activate as view_activate
from creopyson.view import list_exploded as view_list_exploded
from creopyson.view import list_ as view_list
from creopyson.view import save as view_save

Client.view_activate = view_activate
Client.view_list_exploded = view_list_exploded
Client.view_list = view_list
Client.view_save = view_save


# Windchill
from creopyson.windchill import authorize as windchill_authorize
from creopyson.windchill import clear_workspace as windchill_clear_workspace
from creopyson.windchill import create_workspace as windchill_create_workspace
from creopyson.windchill import delete_workspace as windchill_delete_workspace
from creopyson.windchill import file_checked_out as windchill_file_checked_out
from creopyson.windchill import get_workspace as windchill_get_workspace
from creopyson.windchill import list_workspace_files as windchill_list_workspace_files
from creopyson.windchill import list_workspaces as windchill_list_workspaces
from creopyson.windchill import server_exists as windchill_server_exists
from creopyson.windchill import set_server as windchill_set_server
from creopyson.windchill import set_workspace as windchill_set_workspace
from creopyson.windchill import workspace_exists as windchill_workspace_exists

Client.windchill_authorize = windchill_authorize
Client.windchill_clear_workspace = windchill_clear_workspace
Client.windchill_create_workspace = windchill_create_workspace
Client.windchill_delete_workspace = windchill_delete_workspace
Client.windchill_file_checked_out = windchill_file_checked_out
Client.windchill_get_workspace = windchill_get_workspace
Client.windchill_list_workspace_files = windchill_list_workspace_files
Client.windchill_list_workspaces = windchill_list_workspaces
Client.windchill_server_exists = windchill_server_exists
Client.windchill_set_server = windchill_set_server
Client.windchill_set_workspace = windchill_set_workspace
Client.windchill_workspace_exists = windchill_workspace_exists



"""View testing."""
import creopyson
from .fixtures import mk_creoson_post_dict, mk_creoson_post_None, mk_getactivefile


def test_view_activate(mk_creoson_post_None, mk_getactivefile):
    """Test activate."""
    c = creopyson.Client()
    result = c.view_activate(
        "name",
        file_="file.prt"
    )
    assert result is None
    result = c.view_activate("name")
    assert result is None


def test_view_list_exploded(mk_creoson_post_dict, mk_getactivefile):
    """Test list_exploded."""
    c = creopyson.Client()
    result = c.view_list_exploded(
        file_="file",
        name="name"
    )
    assert isinstance(result, (list))
    result = c.view_list_exploded()
    assert isinstance(result, (list))


def test_view_list(mk_creoson_post_dict, mk_getactivefile):
    """Test list."""
    c = creopyson.Client()
    result = c.view_list(
        file_="file",
        name="name"
    )
    assert isinstance(result, (list))
    result = c.view_list()
    assert isinstance(result, (list))


def test_view_save(mk_creoson_post_None, mk_getactivefile):
    """Test save."""
    c = creopyson.Client()
    result = c.view_save(
        "name",
        file_="file.prt"
    )
    assert result is None
    result = c.view_save("name")
    assert result is None



"""Interface testing."""
import pytest
import creopyson
from .fixtures import mk_creoson_post_dict, mk_creoson_post_None, mk_getactivefile


def test_interface_export_3dpdf(mk_creoson_post_dict, mk_getactivefile):
    """Test export_3dpdf."""
    c = creopyson.Client()
    result = c.interface_export_3dpdf(
        file_="file",
        filename="filename",
        dirname="dirname",
        height=12.3,
        width=12.3,
        dpi=200,
        use_drawing_settings=True,
        sheet_range="current",
    )
    assert isinstance(result, (dict))
    result = c.interface_export_3dpdf()
    assert isinstance(result, (dict))


def test_interface_export_file(mk_creoson_post_dict, mk_getactivefile):
    """Test export_file."""
    c = creopyson.Client()
    result = c.interface_export_file(
        "STEP",
        file_="file",
        filename="filename",
        dirname="dirname",
        geom_flags="default",
        advanced=True,
    )
    assert isinstance(result, (dict))
    result = c.interface_export_file("STEP")
    assert isinstance(result, (dict))


def test_interface_export_image(mk_creoson_post_dict, mk_getactivefile):
    """Test export_image."""
    c = creopyson.Client()
    result = c.interface_export_image(
        "JPEG",
        file_="file",
        filename="filename",
        height=9.2,
        width=12.3,
        dpi=200,
        depth=12,
    )
    assert isinstance(result, (dict))
    result = c.interface_export_image("JPEG")
    assert isinstance(result, (dict))


def test_interface_export_pdf(mk_creoson_post_dict, mk_getactivefile):
    """Test export_pdf."""
    c = creopyson.Client()
    result = c.interface_export_pdf(
        file_="file",
        filename="filename",
        dirname="dirname",
        height=12.3,
        width=12.3,
        dpi=200,
        use_drawing_settings=True,
        sheet_range="current",
    )
    assert isinstance(result, (dict))
    result = c.interface_export_pdf()
    assert isinstance(result, (dict))


def test_interface_export_program(mk_creoson_post_dict, mk_getactivefile):
    """Test export_program."""
    c = creopyson.Client()
    result = c.interface_export_program(file_="file")
    assert isinstance(result, (dict))
    result = c.interface_export_program()
    assert isinstance(result, (dict))


def test_import_file(mk_creoson_post_dict):
    """Test import_file."""
    c = creopyson.Client()
    result = c.interface_import_file(
        "filename.stp",
        "STEP",
        dirname="directory/of/filename/",
        new_name="my_imported_file",
        new_model_type="prt",
    )
    assert isinstance(result, (str))


def test_import_file_tryAllExtensions(mk_creoson_post_dict):
    """Test import_file."""
    c = creopyson.Client()
    result = c.interface_import_file("filename.stp")
    assert isinstance(result, (str))
    result = c.interface_import_file("filename.igs.1")
    assert isinstance(result, (str))
    result = c.interface_import_file("filename.neu.59")
    assert isinstance(result, (str))
    result = c.interface_import_file("filename.pvz")
    assert isinstance(result, (str))


def test_import_file_Needfile_type(mk_creoson_post_dict):
    """Test import_file."""
    c = creopyson.Client()
    with pytest.raises(TypeError):
        c.interface_import_file("filename.fake_ext.12")


def test_interface_import_program(mk_creoson_post_dict, mk_getactivefile):
    """Test import_program."""
    c = creopyson.Client()
    result = c.interface_import_program(
        file_="file", filename="filename", dirname="dirname"
    )
    assert isinstance(result, (str))
    result = c.interface_import_program()
    assert isinstance(result, (str))


def test_interface_mapkey(mk_creoson_post_None):
    """Test mapkey."""
    c = creopyson.Client()
    result = c.interface_mapkey("script", delay=12)
    assert result is None


def test_interface_plot(mk_creoson_post_dict, mk_getactivefile):
    """Test plot."""
    c = creopyson.Client()
    result = c.interface_plot(file_="file", dirname="dirname", driver="JPEG")
    assert isinstance(result, (dict))
    result = c.interface_plot()
    assert isinstance(result, (dict))



"""Creo testing."""
import creopyson
from creopyson.exceptions import MissingKey
from .fixtures import mk_creoson_post_dict, mk_creoson_post_None


def test_creo_cd(mk_creoson_post_dict):
    """Test creo_cd."""
    c = creopyson.Client()
    dirname = "C:\\User\\Working directory\\"
    result = c.creo_cd(dirname)
    assert isinstance(result, (str))


def test_creo_delete_files(mk_creoson_post_dict):
    """Test creo_delete_files."""
    c = creopyson.Client()
    result = c.creo_delete_files()
    assert isinstance(result, (list))
    result = c.creo_delete_files(dirname="C:/dir/", filename="fake_file.prt")
    assert isinstance(result, (list))
    result = c.creo_delete_files(filename=["C:/dir/fake_file.prt"])
    assert isinstance(result, (list))


def test_creo_get_config(mk_creoson_post_dict):
    """Test creo_get_config."""
    c = creopyson.Client()
    result = c.creo_get_config("option")
    assert isinstance(result, (list))


def test_creo_get_std_color(mk_creoson_post_dict):
    """Test creo_get_std_color."""
    c = creopyson.Client()
    result = c.creo_get_std_color("letter")
    assert isinstance(result, (dict))


def test_creo_list_dirs(mk_creoson_post_dict):
    """Test creo_list_dirs."""
    c = creopyson.Client()
    result = c.creo_list_dirs("filter_*")
    assert isinstance(result, (list))
    result = c.creo_list_dirs()
    assert isinstance(result, (list))


def test_creo_list_dirs_empty(monkeypatch):
    """Correction issue #1.

    if there is no folder in directory, creoson does not return `data`.
    Need to return an empty list.
    """

    def fake_func(client, command, function, data=None, key_data=None):
        raise MissingKey("Missing `data` in creoson return")

    monkeypatch.setattr(creopyson.connection.Client, "_creoson_post", fake_func)
    c = creopyson.Client()
    result = c.creo_list_dirs()
    assert result == []


def test_creo_list_files(mk_creoson_post_dict):
    """Test creo_list_dirs."""
    c = creopyson.Client()
    result = c.creo_list_files("filter_*")
    assert isinstance(result, (list))


def test_creo_mkdir(mk_creoson_post_dict):
    """Test creo_mkdir."""
    c = creopyson.Client()
    result = c.creo_mkdir("new_dir")
    assert isinstance(result, (str))


def test_creo_pwd(mk_creoson_post_dict):
    """Test creo_pwd."""
    c = creopyson.Client()
    result = c.creo_pwd()
    assert isinstance(result, (str))


def test_creo_rmdir_ok(mk_creoson_post_None):
    """Test creo_rmdir."""
    c = creopyson.Client()
    result = c.creo_rmdir("fake_dir")
    assert result is None


def test_creo_set_config_ok(mk_creoson_post_None):
    """Test creo_set_config."""
    c = creopyson.Client()
    result = c.creo_set_config("option", 12, True)
    assert result is None


def test_creo_set_creo_version(mk_creoson_post_None):
    """Test creo_set_creao_version."""
    c = creopyson.Client()
    result = c.creo_set_creo_version(7)
    assert result is None


def test_creo_set_std_color_ok(mk_creoson_post_None):
    """Test creo_set_std_color."""
    c = creopyson.Client()
    result = c.creo_set_std_color("letter", 0, 0, 0)
    assert result is None



"""Objects Module."""


def jlpoint(x, y, z):
    """Return a 3D coordinate dict.

    Args:
        x (float):
            X-coordinate.
        y (float):
            Y-coordinate.
        z (float):
            Z-coordinate.

    Returns:
        (dict): 3D coordinate object.

    """
    try:
        x, y, z = float(x), float(y), float(z)
    except ValueError:
        raise Warning("Coordonates must be numbers")
    return {
        "x": x,
        "y": y,
        "z": z
    }



"""Parameter module."""


def copy(client, name, to_name, file_=None, to_file=None, designate=None):
    """Copy parameter to another in the same model or another model.

    Args:
        client (obj):
            creopyson Client.
        name (str):
            Parameter name to copy (wildcards allowed: True).
        to_name (str):
            Destination parameter.
        `file_` (str, optional):
            Model name. Defaults is current active model.
        to_file (str, optional):
            Destination model (wildcards allowed: True).
            Defaults is the source model.
        designate (boolean, optional):
            Set copied parameter to be designated/not designated,
            blank=do not set. Defaults is `blank`.

    Returns:
        None

    """
    data = {
        "name": name,
        "to_name": to_name,
    }
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    if to_file is not None:
        data["to_file"] = to_file
    if designate is not None:
        data["designate"] = designate
    return client._creoson_post("parameter", "copy", data)


def delete(client, name, file_=None):
    """Delete a parameter.

    Args:
        client (obj):
            creopyson Client.
        name (str):
            Parameter name (wildcards allowed: True).
        `file_` (str, optional):
            Model name. Defaults is current active model.

    Returns:
        None

    """
    data = {"name": name}
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    return client._creoson_post("parameter", "delete", data)


def exists(client, name=None, file_=None):
    """Check whether parameter(s) exists on a model.

    Args:
        client (obj):
            creopyson Client.
        name (str|list:str, optional):
            Parameter name; List of parameter names.
            if empty it checks for any parameter's existence.
        `file_` (str, optional):
            Model name. Defaults is current active model.

    Returns:
        (boolean): Whether the parameter exists on the model.

    """
    data = {}
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    if name is not None:
        if isinstance(name, (str)):
            data["name"] = name
        elif isinstance(name, (list)):
            data["names"] = name
    return client._creoson_post("parameter", "exists", data, "exists")


def list_(client, name=None, file_=None, encoded=None, value=None):
    """Get a list of parameters from one or more models.

    Args:
        client (obj):
            creopyson Client.
        name (str|list:str, optional):
            Parameter name; List of parameter names.
            if empty it checks for any parameter's existence.
        `file_` (str, optional):
            Model name. Defaults is current active model.
        encoded (boolean, optional):
            Whether to return the values Base64-encoded. Defaults is False.
        value (str, optional):
            Parameter value filter. Defaults is `no filter`.

    Returns:
        (list:dict):
            name (str): Parameter name.
            type (str): Parameter type.
            value (various): Parameter value # TODO
            designate (boolean): Whether the parameter is designated.
            description (str): Description.
            encoded (boolean): Whether the parameter is encoded.
            owner_name (str): File name.

    """
    data = {}
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    if name is not None:
        if isinstance(name, (str)):
            data["name"] = name
        elif isinstance(name, (list)):
            data["names"] = name
    if encoded is not None:
        data["encoded"] = encoded
    if value is not None:
        data["value"] = value
    return client._creoson_post("parameter", "list", data, "paramlist")


def set_(
    client,
    name,
    value=None,
    file_=None,
    type_=None,
    encoded=None,
    designate=None,
    description=None,
    no_create=None,
):
    """Set the value of a parameter.

    Args:
        client (obj):
            creopyson Client.
        name (str):
            Parameter name (wildcards allowed: True).
        value (depends on data type, optional):
            Parameter value. Defaults to None.
            Clears the parameter value if missing.
        `file_` (str, optional):
            Model name. Defaults is current active model.
        `type_` (str, optional):
            Data type. Defaults is `STRING`.
            Valid values: STRING, DOUBLE, INTEGER, BOOL, NOTE.
        encoded (boolean, optional):
            Whether the value is Base64-encoded. Defaults is False.
        designate (boolean, optional):
            Set parameter to be designated/not designated, blank=do not set.
            Defaults is `blank`.
        description (str, optional):
            Parameter description.
            If missing, leaves the current description in place.
        no_create (boolean, optional):
            If parameter does not already exist, do not create it.
            Defaults is False.

    Returns:
        None

    """
    data = {"name": name}
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    if type_ is not None:
        data["type"] = type_
    if encoded is not None:
        data["encoded"] = encoded
    if value is not None:
        data["value"] = value
    if designate is not None:
        data["designate"] = designate
    if description is not None:
        data["description"] = description
    if no_create is not None:
        data["no_create"] = no_create
    return client._creoson_post("parameter", "set", data)


def set_designated(
    client,
    name,
    designate,
    file_=None,
):
    """Set the designated state of a parameter.

    Args:
        client (obj):
            creopyson Client.
        name (str):
            Parameter name (wildcards allowed: True).
        designate (boolean):
            Set parameter to be designated/not designated.
        `file_` (str, optional):
            Model name. Defaults is current active model.

    Returns:
        None

    """
    data = {
        "name": name,
        "designate": designate,
    }
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    return client._creoson_post("parameter", "set_designated", data)



"""Note testing."""
import creopyson
from .fixtures import mk_creoson_post_dict, mk_creoson_post_None, mk_getactivefile


def test_note_copy(mk_creoson_post_None, mk_getactivefile):
    """Test copy."""
    c = creopyson.Client()
    result = c.note_copy(
        "name", to_name="target name", file_="file", to_file="target file"
    )
    assert result is None
    result = c.note_copy("name")
    assert result is None


def test_note_delete(mk_creoson_post_None, mk_getactivefile):
    """Test delete."""
    c = creopyson.Client()
    result = c.note_delete("name", file_="file")
    assert result is None
    result = c.note_delete("name")
    assert result is None


def test_note_exists(mk_creoson_post_dict, mk_getactivefile):
    """Test exists."""
    c = creopyson.Client()
    result = c.note_exists(file_="file", name="name")
    assert isinstance(result, (bool))
    result = c.note_exists(name=["name", "other name"])
    assert isinstance(result, (bool))


def test_note_get(mk_creoson_post_dict, mk_getactivefile):
    """Test get."""
    c = creopyson.Client()
    result = c.note_get("name", file_="file")
    assert isinstance(result, (dict))
    result = c.note_get("name")
    assert isinstance(result, (dict))


def test_note_list(mk_creoson_post_dict, mk_getactivefile):
    """Test list."""
    c = creopyson.Client()
    result = c.note_list(
        file_="file", name="name", value="value", get_expanded=True, select=True
    )
    assert isinstance(result, (list))
    result = c.note_list(name=["name", "other name"])
    assert isinstance(result, (list))


def test_note_set(mk_creoson_post_None, mk_getactivefile):
    """Test set."""
    c = creopyson.Client()
    result = c.note_set(
        "name",
        file_="file",
        location={"x": 0, "y": 0, "z": 0},
        encoded=True,
        value="value",
    )
    assert result is None
    result = c.note_set("name")
    assert result is None



"""Drawing testing."""
import creopyson
from .fixtures import mk_creoson_post_dict, mk_creoson_post_None


def test_drawing_add_model(mk_creoson_post_None):
    """Test add_model."""
    c = creopyson.Client()
    result = c.drawing_add_model("model", drawing="drawing")
    assert result is None


def test_drawing_add_sheet(mk_creoson_post_None):
    """Test add_sheet."""
    c = creopyson.Client()
    result = c.drawing_add_sheet(position=3, drawing="drawing")
    assert result is None


def test_drawing_create(mk_creoson_post_dict):
    """Test create."""
    c = creopyson.Client()
    result = c.drawing_create(
        "template",
        model="model",
        drawing="drawing",
        scale=2.5,
        display=True,
        activate=True,
        new_window=True
    )
    assert isinstance(result, (str))


def test_drawing_create_gen_view(mk_creoson_post_None):
    """Test create_gen_view."""
    c = creopyson.Client()
    result = c.drawing_create_gen_view(
        "model_view",
        {"point": 12},
        drawing="drawing",
        view="view",
        sheet=3,
        model="model",
        scale=1.5,
        display_data={"display": "data"},
        exploded=True
    )
    assert result is None


def test_drawing_create_proj_view(mk_creoson_post_None):
    """Test create_proj_view."""
    c = creopyson.Client()
    result = c.drawing_create_proj_view(
        "parent_view",
        {"point": "xyz"},
        drawing="drawing",
        view="view",
        sheet=2,
        display_data=True,
        exploded=True
    )
    assert result is None


def test_drawing_create_symbol(mk_creoson_post_None):
    """Test create_symbol."""
    c = creopyson.Client()
    result = c.drawing_create_symbol(
        "symbol_file",
        {"point": "xyz"},
        drawing="drawing",
        replace_values={"replace": "values"},
        sheet=3
    )
    assert result is None


def test_drawing_delete_models(mk_creoson_post_None):
    """Test delete_models."""
    c = creopyson.Client()
    result = c.drawing_delete_models(
        model="model",
        drawing="drawing",
        delete_views=True
    )
    assert result is None


def test_drawing_delete_sheet(mk_creoson_post_None):
    """Test delete_sheet."""
    c = creopyson.Client()
    result = c.drawing_delete_sheet(5, drawing="drawing")
    assert result is None


def test_drawing_delete_symbol_def(mk_creoson_post_None):
    """Test delete_symbol_def."""
    c = creopyson.Client()
    result = c.drawing_delete_symbol_def("symbol_file", drawing="drawing")
    assert result is None


def test_drawing_delete_symbol_inst(mk_creoson_post_None):
    """Test delete_symbol_inst."""
    c = creopyson.Client()
    result = c.drawing_delete_symbol_inst("symbol_id", drawing="drawing")
    assert result is None


def test_drawing_delete_view(mk_creoson_post_None):
    """Test delete_view."""
    c = creopyson.Client()
    result = c.drawing_delete_view(
        "view",
        drawing="drawing",
        sheet=3,
        del_children=True
    )
    assert result is None


def test_drawing_get_cur_model(mk_creoson_post_dict):
    """Test get_cur_model."""
    c = creopyson.Client()
    result = c.drawing_get_cur_model(drawing="drawing")
    assert isinstance(result, (str))


def test_drawing_get_cur_sheet(mk_creoson_post_dict):
    """Test get_cur_sheet."""
    c = creopyson.Client()
    result = c.drawing_get_cur_sheet(drawing="drawing")
    assert isinstance(result, (int))


def test_drawing_get_num_sheets(mk_creoson_post_dict):
    """Test get_num_sheets."""
    c = creopyson.Client()
    result = c.drawing_get_num_sheets(drawing="drawing")
    assert isinstance(result, (int))


def test_drawing_get_sheet_format(mk_creoson_post_dict):
    """Test get_sheet_format."""
    c = creopyson.Client()
    result = c.drawing_get_sheet_format(1, drawing="drawing")
    assert isinstance(result, (dict))


def test_drawing_get_sheet_scale(mk_creoson_post_dict):
    """Test get_sheet_scale."""
    c = creopyson.Client()
    result = c.drawing_get_sheet_scale(
        2,
        drawing="drawing",
        model="model"
    )
    assert isinstance(result, (float))


def test_drawing_get_sheet_size(mk_creoson_post_dict):
    """Test get_sheet_size."""
    c = creopyson.Client()
    result = c.drawing_get_sheet_size(2, drawing="drawing")
    assert isinstance(result, (str))


def test_drawing_get_view_loc(mk_creoson_post_dict):
    """Test get_view_loc."""
    c = creopyson.Client()
    result = c.drawing_get_view_loc("view", drawing="drawing")
    assert isinstance(result, (dict))


def test_drawing_get_view_scale(mk_creoson_post_dict):
    """Test get_view_scale."""
    c = creopyson.Client()
    result = c.drawing_get_view_scale("view", drawing="drawing")
    assert isinstance(result, (float))


def test_drawing_get_view_sheet(mk_creoson_post_dict):
    """Test get_view_sheet."""
    c = creopyson.Client()
    result = c.drawing_get_view_sheet("view", drawing="drawing")
    assert isinstance(result, (int))


def test_drawing_is_symbol_def_loaded(mk_creoson_post_dict):
    """Test is_symbol_def_loaded."""
    c = creopyson.Client()
    result = c.drawing_is_symbol_def_loaded("symbol_file", drawing="drawing")
    assert isinstance(result, (bool))


def test_drawing_list_models(mk_creoson_post_dict):
    """Test list_models."""
    c = creopyson.Client()
    result = c.drawing_list_models(model="model", drawing="drawing")
    assert isinstance(result, (list))


def test_drawing_list_symbols(mk_creoson_post_dict):
    """Test list_symbols."""
    c = creopyson.Client()
    result = c.drawing_list_symbols(
        drawing="drawing",
        symbol_file="symbol_file",
        sheet=2
    )
    assert isinstance(result, (list))


def test_drawing_list_view_details(mk_creoson_post_dict):
    """Test list_view_details."""
    c = creopyson.Client()
    result = c.drawing_list_view_details(view="view", drawing="drawing")
    assert isinstance(result, (list))


def test_drawing_list_views(mk_creoson_post_dict):
    """Test list_views."""
    c = creopyson.Client()
    result = c.drawing_list_views(view="view", drawing="drawing")
    assert isinstance(result, (list))


def test_drawing_load_symbol_def(mk_creoson_post_dict):
    """Test load_symbol_def."""
    c = creopyson.Client()
    result = c.drawing_load_symbol_def(
        "symbol_file", symbol_dir="symbol_dir", drawing="drawing")
    assert isinstance(result, (dict))


def test_drawing_regenerate(mk_creoson_post_None):
    """Test regenerate."""
    c = creopyson.Client()
    result = c.drawing_regenerate(drawing="drawing")
    assert result is None


def test_drawing_regenerate_sheet(mk_creoson_post_None):
    """Test regenerate_sheet."""
    c = creopyson.Client()
    result = c.drawing_regenerate_sheet(sheet=2, drawing="drawing")
    assert result is None


def test_drawing_rename_view(mk_creoson_post_None):
    """Test rename_view."""
    c = creopyson.Client()
    result = c.drawing_rename_view("old_name", "new_name", drawing="drawing")
    assert result is None


def test_drawing_scale_sheet(mk_creoson_post_None):
    """Test scale_sheet."""
    c = creopyson.Client()
    result = c.drawing_scale_sheet(2, 1.5, drawing="drawing", model="model")
    assert result is None


def test_drawing_scale_view(mk_creoson_post_dict):
    """Test scale_view."""
    c = creopyson.Client()
    result = c.drawing_scale_view(1.5, drawing="drawing", view="view")
    assert isinstance(result, (dict))


def test_drawing_select_sheet(mk_creoson_post_None):
    """Test select_sheet."""
    c = creopyson.Client()
    result = c.drawing_select_sheet(3, drawing="drawing")
    assert result is None


def test_drawing_set_cur_model(mk_creoson_post_None):
    """Test set_cur_model."""
    c = creopyson.Client()
    result = c.drawing_set_cur_model("model", drawing="drawing")
    assert result is None


def test_drawing_set_sheet_formatl(mk_creoson_post_None):
    """Test set_sheet_format."""
    c = creopyson.Client()
    result = c.drawing_set_sheet_format(1, "file_fmt", dirname="dirname", drawing="drawing")
    assert result is None


def test_drawing_set_view_loc(mk_creoson_post_None):
    """Test set_view_loc."""
    c = creopyson.Client()
    result = c.drawing_set_view_loc(
        "view", {"coord": "xyz"}, drawing="drawing")
    assert result is None


def test_drawing_view_bound_box(mk_creoson_post_dict):
    """Test view_bound_box."""
    c = creopyson.Client()
    result = c.drawing_view_bound_box("view", drawing="drawing")
    assert isinstance(result, (dict))



"""Layer testing."""
import creopyson
from .fixtures import mk_creoson_post_dict, mk_creoson_post_None, mk_getactivefile


def test_layer_delete(mk_creoson_post_None, mk_getactivefile):
    """Test delete."""
    c = creopyson.Client()
    result = c.layer_delete(name="name", file_="file")
    assert result is None
    result = c.layer_delete()
    assert result is None


def test_layer_exists(mk_creoson_post_dict, mk_getactivefile):
    """Test exists."""
    c = creopyson.Client()
    result = c.layer_exists(name="name", file_="file")
    assert isinstance(result, (bool))
    result = c.layer_exists()
    assert isinstance(result, (bool))


def test_layer_list(mk_creoson_post_dict, mk_getactivefile):
    """Test list."""
    c = creopyson.Client()
    result = c.layer_list(name="name", file_="file")
    assert isinstance(result, (list))
    result = c.layer_list()
    assert isinstance(result, (list))


def test_layer_show(mk_creoson_post_None, mk_getactivefile):
    """Test show."""
    c = creopyson.Client()
    result = c.layer_show(name="name", file_="file", show_=True)
    assert result is None
    result = c.layer_show()
    assert result is None



"""Dimension module."""


def copy(client, name, to_name, file_=None, to_file=None):
    """Copy dimension to another in the same model or another model.

    Args:
        client (obj):
            creopyson Client.
        name (str):
            Dimension name to copy.
        to_name (str):
            Destination dimension; th dimension must already exist.
        `file_` (str, optional):
            Model name. Defaults is current active model.
        to_file (str, optional):
            Destination model. Defaults is the source model.

    Returns:
        None

    """
    data = {"name": name, "to_name": to_name}
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    if to_file is not None:
        data["to_file"] = to_file
    return client._creoson_post("dimension", "copy", data)


def list_(client, name=None, file_=None, dim_type=None, encoded=None, select=False):
    """Get a list of dimensions from a model.

    If select is true, then the current selection in Creo will be cleared even
    if no items are found.

    Args:
        client (obj):
            creopyson Client.
        name (str|list:str, optional):
            Dimension name;
            if empty then all dimensions are listed.
        `file_` (str, optional):
            Model name. Defaults is current active model.
        dim_type (str, optional):
            Dimension type filter. Defaults is `no filter`.
            Valid values: linear, radial, diameter, angular.
        encoded (boolean, optional):
            Whether to return the values Base64-encoded. Defaults is False.
        select (boolean, optional):
            If true, the dimensions that are found will be selected in Creo.
            Defaults is False.

    Returns:
        (list:dict): List of dimension information.
            name (str):
                Dimension name
            value (str|float):
                Dimension value; if encoded is True it is a str,
                if encoded is False it is a float.
            encoded (boolean):
                Whether the returned value is Base64-encoded.
            dwg_dim (boolean):
                Whether dimension is a drawing dimension rather than
                a model dimension.

    """
    data = {}
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    if name is not None:
        if isinstance(name, (str)):
            data["name"] = name
        elif isinstance(name, (list)):
            data["names"] = name
    if dim_type is not None:
        data["dim_type"] = dim_type
    if encoded is not None:
        data["encoded"] = encoded
    if select is not None:
        data["select"] = select
    return client._creoson_post("dimension", "list", data, "dimlist")


def list_detail(
    client, name=None, file_=None, dim_type=None, encoded=None, select=False
):
    """Get a list of dimension details from a model.

    Values will automatically be returned Base64-encoded if they are strings
    which contain Creo Symbols or other non-ASCII data.
    If select is true, then the current selection in Creo will be cleared
    even if no items are found.

    Args:
        client (obj):
            creopyson Client.
        name (str|list:str, optional):
            Dimension name;
            if empty then all dimensions are listed.
        `file_` (str, optional):
            Model name. Defaults is current active model.
        dim_type (str, optional):
            Dimension type filter. Defaults is `no filter`.
            Valid values: linear, radial, diameter, angular.
        encoded (boolean, optional):
            Whether to return the values Base64-encoded. Defaults is False.
        select (boolean, optional):
            If true, the dimensions that are found will be selected in Creo.
            Defaults is False.

    Returns:
        (list:dict): List of dimension information.
            name (str):
                Dimension name
            value (str|float):
                Dimension value; if encoded is True it is a str,
                if encoded is False it is a float.
            encoded (boolean):
                Whether the returned value is Base64-encoded.
            sheet (int):
                Sheet number.
            view_name (str):
                View name.
            dim_type (str):
                Dimension type.
                Valid values: linear, radial, diameter, angular.
            dwg_dim (boolean):
                Whether dimension is a drawing dimension rather than
                a model dimension.
            text (str):
                dimension text.
            location (dict): Coordonates location.
                x (float): X coordonate location.
                y (float): Y coordonate location.
                z (float): Z coordonate location.
            tolerance_type (str):
                Tolerance type, if not specified not returned.
                Valid values: plus_minus (TODO complete list).
            tol_plus (float):
                Plus tolerance value.
                if tolerance_type not specified not returned.
            tol_minus (float):
                Minus tolerance value.
                if tolerance_type not specified not returned.

    """
    data = {}
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    if name is not None:
        if isinstance(name, (str)):
            data["name"] = name
        elif isinstance(name, (list)):
            data["names"] = name
    if dim_type is not None:
        data["dim_type"] = dim_type
    if encoded is not None:
        data["encoded"] = encoded
    if select is not None:
        data["select"] = select
    return client._creoson_post("dimension", "list_detail", data, "dimlist")


def set_(client, name, value, file_=None, encoded=None):
    r"""Set a dimension value.

    One reason to encode values is if the value contains special characters,
    such as Creo symbols.
    You may be able to avoid Base64-encoding symbols by using Unicode for the
    binary characters, for example including \\u0001#\\u0002 in the value to
    insert a plus/minus symbol.

    Args:
        client (obj):
            creopyson Client.
        name (str):
            Dimension name.
        value (str|float):
            Dimension value.
        `file_` (string, optional):
            file name, if not set, active model is used.
        encoded (boolean, optional):
            Whether the value is Base64-encoded.
            Defaults is False.


    Raises:
        Warning: error message from creoson.

    Returns:
        None

    """
    data = {
        "name": name,
        "value": value,
    }
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    if encoded is not None:
        data["encoded"] = encoded
    return client._creoson_post("dimension", "set", data)


def set_text(client, name, file_=None, text=None, encoded=False):
    """Set dimension text.

    Args:
        client (obj):
            creopyson object.
        name (str):
            Dimension name.
        `file_` (string, optional):
            file name, if not set, active model is used.
        text ([type], optional):
            Dimension text. Defaults to None, sets the dimension's text to @D.
        encoded (bool, optional):
            Whether the text value is Base64-encoded. Defaults to False.

    Returns:
        None

    """
    data = {
        "name": name,
        "encoded": encoded,
    }
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    if text is not None:
        data["text"] = text
    return client._creoson_post("dimension", "set_text", data)


def show(client, name, file_=None, assembly=None, path=None):
    """Display or hide a dimension in Creo.

    Args:
        client (obj):
            creopyson Client.
        name (str):
            Dimension name.
        `file_` (str, optional):
            Model name. Defaults is current active model.
        assembly (str, optional):
            Assembly name; only used if path is given.
            Defaults is the currently active model.
        path (list:int, optional):
            Path to occurrence of the model within the assembly;
            the dimension will only be shown for that occurrence.
            Defaults: all occurrences of the component are affected.

    Returns:
        None

    """
    data = {"name": name}
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    if assembly is not None:
        data["assembly"] = assembly
    if path is not None:
        data["path"] = path
    return client._creoson_post("dimension", "show", data)


def user_select(client, file_=None, maxi=None):
    """Prompt user to select one or more dimensions, and return their selections.

        client (obj):
            creopyson Client.
        `file_` (str, optional):
            Model name. Defaults is current active model.
        maxi (int, optional):
            The maximum number of dimensions that the user can select.
            Defaults is `1`.

    Raises:
        Warning: error message from creoson.

    Returns:
        (list:dict): List of selected dimension information
            name (str):
                Dimension name
            value (str|float):
                Dimension value; if encoded is True it is a str,
                if encoded is False it is a float.
            encoded (boolean):
                Whether the returned value is Base64-encoded.
            file (str):
                File name.
            relation_id (int):
                Relation ID number.

    """
    data = {"max": 1}
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    if maxi is not None:
        data["max"] = maxi
    return client._creoson_post("dimension", "user_select", data, "dimlist")



"""Dimension testing."""
import creopyson
from .fixtures import mk_creoson_post_dict, mk_creoson_post_None, mk_getactivefile


def test_dimension_copy(mk_creoson_post_None, mk_getactivefile):
    """Test dimension_copy."""
    c = creopyson.Client()
    result = c.dimension_copy(
        "old_name",
        "new_name",
        file_="file.prt",
        to_file="file.prt"
    )
    assert result is None
    result = c.dimension_copy(
        "old_name",
        "new_name"
    )
    assert result is None


def test_dimension_list(mk_creoson_post_dict, mk_getactivefile):
    """Test dimension_list."""
    c = creopyson.Client()
    result = c.dimension_list(
        name="name",
        file_="file.prt",
        dim_type="linear",
        encoded=True,
        select=True
    )
    assert isinstance(result, (list))
    result = c.dimension_list(
        name=["name", "other name"],
        file_="file.prt",
        dim_type="linear",
        encoded=True
    )
    assert isinstance(result, (list))
    result = c.dimension_list()
    assert isinstance(result, (list))


def test_dimension_list_detail(mk_creoson_post_dict, mk_getactivefile):
    """Test dimension_list."""
    c = creopyson.Client()
    result = c.dimension_list_detail(
        name="name",
        file_="file.prt",
        dim_type="linear",
        encoded=True,
        select=True
    )
    assert isinstance(result, (list))
    result = c.dimension_list_detail(
        name=["name", "other name"],
        file_="file.prt",
        dim_type="linear",
        encoded=True
    )
    assert isinstance(result, (list))
    result = c.dimension_list_detail()
    assert isinstance(result, (list))


def test_dimension_set(mk_creoson_post_None, mk_getactivefile):
    """Test dimension_set."""
    c = creopyson.Client()
    result = c.dimension_set(
        "name",
        12.4,
        file_="file",
        encoded=True
    )
    assert result is None
    result = c.dimension_set(
        "name",
        12.4,
    )
    assert result is None


def test_dimension_set_text(mk_creoson_post_None, mk_getactivefile):
    """Test dimension_set_text."""
    c = creopyson.Client()
    result = c.dimension_set_text(
        "name",
        text="@D rad",
        file_="file"
    )
    assert result is None
    result = c.dimension_set_text(
        "name",
        encoded=True
    )
    assert result is None


def test_dimension_show(mk_creoson_post_None, mk_getactivefile):
    """Test dimension_show."""
    c = creopyson.Client()
    result = c.dimension_show(
        "name",
        file_="filename",
        assembly="asm name",
        path=['path']
    )
    assert result is None
    result = c.dimension_show("name")
    assert result is None


def test_dimension_user_select(mk_creoson_post_dict, mk_getactivefile):
    """Test user_select."""
    c = creopyson.Client()
    result = c.dimension_user_select(
        file_="file.prt",
        maxi=3
    )
    assert isinstance(result, (list))
    result = c.dimension_user_select()
    assert isinstance(result, (list))



"""Interface module.

Export 3D pdf, file (step, iges, dxf, etc...), picture, plot
Run mapkey
Import/Export program (pls, als)

"""
import re


def export_3dpdf(
    client,
    file_=None,
    filename=None,
    dirname=None,
    height=None,
    width=None,
    dpi=None,
    use_drawing_settings=None,
    sheet_range="all",
):
    """Export a model to a 3D PDF file.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str, optional):
            Model name. Defaults is current active model.
        filename (str, optional):
            Destination file name. May also contain a path to the file.
            Defaults is the model name with the appropriate file extension,
            in Creo's working directory.
        dirname (str, optional):
            Destination directory. Defaults is Creo's current
            working directory.
        height (float, optional):
            PDF image height. Defaults is Creo default export height.
        width (float, optional):
            PDF image width. Defaults is Creo default export width.
        dpi (int, optional):
            PDF Image DPI. Default is Creo default export DPI.
        use_drawing_settings (boolean, optional):
            Whether to use special settings for exporting drawings.
            Defaut is False.
        sheet_range (string):
            Range of drawing sheets to export. Default vale is "all".
            Valid values: "all", "current", range of sheet numbers (ex: "1,3-4")

    Returns:
        dict:
            dirname (str): Directory of the output file
            filename (str): Name of the output file

    """
    data = {"sheet_range": sheet_range}
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    if filename is not None:
        data["filename"] = filename
    if dirname is not None:
        data["dirname"] = dirname
    if height is not None:
        data["height"] = height
    if width is not None:
        data["width"] = width
    if dpi is not None:
        data["dpi"] = dpi
    if use_drawing_settings is not None:
        data["use_drawing_settings"] = use_drawing_settings
    return client._creoson_post("interface", "export_3dpdf", data)


def export_file(
    client,
    file_type,
    file_=None,
    filename=None,
    dirname=None,
    geom_flags=None,
    advanced=None,
):
    """Export a model to a file.

    The geom_flags option only applies to IGES and STEP exports.
    Setting geom_flags to 'default' will cause it to check the Creo config
    option 'intf3d_out_default_option' for the setting
    The advanced option will cause the Export to use settings defined in the
    appropriate `export_profiles` Creo Config Option for the file type.
    The advanced option only applies to DXF, IGES and STEP exports.
    The advanced option will only work with Creo 4 M030 or later.

    Args:
        client (obj):
            creopyson Client.
        file_type (str):
            File type.
            Valid values: "DXF", "IGES", "NEUTRAL", "PV", "STEP", "VRML".
        `file_` (str, optional):
            Model name. Defaults is current active model.
        filename (str, optional):
            Destination file name. May also contain a path to the file.
            Defaults is the model name with the appropriate file extension,
            in Creo's working directory.
        dirname (str, optional):
            Destination directory. Defaults is Creo's current
            working directory.
        geom_flags (str, optional):
            Geometry type for the export.
            Defaults is `solids`.
            Valid values: solids, surfaces, wireframe, wireframe_surfaces,
            quilts, default.
        advanced (Boolean, optional):
            Whether to use the newer Creo 4 file export function.
            Defaults is False.

    Returns:
        dict:
            dirname (str): Directory of the output file
            filename (str): Name of the output file

    """
    data = {"type": file_type}
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    if filename is not None:
        data["filename"] = filename
    if dirname is not None:
        data["dirname"] = dirname
    if geom_flags is not None:
        data["geom_flags"] = geom_flags
    if advanced is not None:
        data["advanced"] = advanced
    return client._creoson_post("interface", "export_file", data)


def export_image(
    client,
    file_type,
    file_=None,
    filename=None,
    height=None,
    width=None,
    dpi=None,
    depth=None,
):
    """Export a model to an image file.

    Args:
        client (obj):
            creopyson Client.
        file_type (str):
            Image Type. Valid values: BMP, EPS, JPEG, TIFF.
        `file_` (str, optional):
            Model name. Defaults is current active model.
        filename (str, optional):
            Destination file name. May also contain a path to the file.
            Defaults is the model name with the appropriate file extension,
            in Creo's working directory.
        height (float, optional):
            Image height. Defaults is `10.0`.
        width (float, optional):
            Image width. Defaults is `7.5`.
        dpi (int, optional):
            Image DPI. Defaults is `100`.
            Valid values: 100, 200, 300, 400, 500, 600.
        depth (int, optional):
            Image depth. Defaults is `24`.
            Valid values: 8, 24.

    Returns:
        dict:
            dirname (str): Directory of the output file
            filename (str): Name of the output file

    """
    data = {"type": file_type}
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    if filename is not None:
        data["filename"] = filename
    # if dirname:
    #     data["dirname"] = dirname
    if height is not None:
        data["height"] = height
    if width is not None:
        data["width"] = width
    if dpi is not None:
        data["dpi"] = dpi
    if depth is not None:
        data["depth"] = depth
    return client._creoson_post("interface", "export_image", data)


def export_pdf(
    client,
    file_=None,
    filename=None,
    dirname=None,
    height=None,
    width=None,
    dpi=None,
    use_drawing_settings=None,
    sheet_range="all",
):
    """Export a model to a PDF file.

    When use_drawing_settings is true, the Font Stroke option will be set to
    Stroke All Fonts, and the Color Depth option will be set to Grayscale.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str, optional):
            Model name. Defaults is current active model.
        filename (str, optional):
            Destination file name. May also contain a path to the file.
            Defaults is the model name with the appropriate file extension,
            in Creo's working directory.
        dirname (str, optional):
            Destination directory. Defaults is Creo's current
            working directory.
        height (float, optional):
            PDF image height. Defaults is Creo default export height.
        width (float, optional):
            PDF image width. Defaults is Creo default export width.
        dpi (int, optional):
            PDF Image DPI. Default is Creo default export DPI.
        use_drawing_settings (boolean, optional):
            Whether to use special settings for exporting drawings.
            Defaut is False.
        sheet_range (string):
            Range of drawing sheets to export. Default vale is "all".
            Valid values: "all", "current", range of sheet numbers (ex: "1,3-4")

    Returns:
        dict:
            dirname (str): Directory of the output file
            filename (str): Name of the output file

    """
    data = {"sheet_range": sheet_range}
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    if filename is not None:
        data["filename"] = filename
    if dirname is not None:
        data["dirname"] = dirname
    if height is not None:
        data["height"] = height
    if width is not None:
        data["width"] = width
    if dpi is not None:
        data["dpi"] = dpi
    if use_drawing_settings is not None:
        data["use_drawing_settings"] = use_drawing_settings
    return client._creoson_post("interface", "export_pdf", data)


def export_program(client, file_=None):
    """Export a model's program to a file.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str, optional):
            Model name. Defaults is current active model.

    Returns:
        dict:
            dirname (str): Directory of the output file
            filename (str): Name of the output file

    """
    data = {}
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    return client._creoson_post("interface", "export_program", data)


def import_file(
    client, filename, file_type=None, dirname=None, new_name=None, new_model_type="asm"
):
    """Import a file as a model.

    Note: This function will not automatically display or activate
    the imported model. If you want that, you should take the file name
    returned by this function and pass it to file:open.
    Users of the old import_pv function should start using this
    function instead.

    Args:
        client (obj):
            creopyson Client.
        filename (str):
            Source file name.
        file_type (str, optional):
            File type.
            Valid values: "IGES", "NEUTRAL", "PV", "STEP".
            Defaults to None. Will analyse filename extension
            *.igs*|*.iges* => IGES
            *.stp*|*.step* => STEP
            *.neu* => NEUTRAL
            *.pvz* => PV
        dirname (str, optional):
            Source directory.
            Defaults is Creo's current working directory.
        new_name (str, optional):
            New model name. Any extension will be stripped off and replaced
            with one based on new_model_type.
            Defaults to None is the name of the file with an extension based
            on new_model_type..
        new_model_type (str, optional):
            New model type.
            Valid values: "asm", "prt"
            Defaults to "asm".

    Returns:
        str: Name of the model imported

    """
    data = {"filename": filename, "new_model_type": new_model_type}

    if file_type is None:
        if re.search(r".*\.(igs|iges).*", filename):
            data["type"] = "IGES"
        elif re.search(r".*\.(stp|step).*", filename):
            data["type"] = "STEP"
        elif re.search(r".*\.(neu).*", filename):
            data["type"] = "NEUTRAL"
        elif re.search(r".*\.(pvz).*", filename):
            data["type"] = "PV"
        else:
            raise TypeError(
                f"`{filename}` extension was not recognized, fill in file_type."
            )
    else:
        data["type"] = file_type

    if dirname is not None:
        data["dirname"] = dirname
    if new_name is not None:
        data["new_name"] = new_name
    return client._creoson_post("interface", "import_file", data, "file")


def import_program(client, file_=None, filename=None, dirname=None):
    """Import a program file for a model.

    Cannot specify both file and filename parameters.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str, optional):
            Destination Model name.
            Defaults is currently active model, or the model for
            the filename parameter if given.
        filename (str, optional):
            Source file name.
            Default is the model name with the appropriate file extension.
        dirname (str, optional):
            Source directory. Defaults is Creo's current working directory.

    Returns:
        str: Name of the model updated

    """
    data = {}
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    if filename is not None:
        data["filename"] = filename
    if dirname is not None:
        data["dirname"] = dirname
    return client._creoson_post("interface", "import_program", data, "file")


def mapkey(client, script, delay=0):
    """Run a Mapkey script in Creo.

    Make sure to remove any `mapkey(continued)` clauses from the script
    argument. The tilde at the start of a line should occur immediately
    after the semicolon at the end of the previous line.

    Args:
        client (obj):
            creopyson Client.
        script (str):
            The mapkey script to run.
        delay (int):
            Amount of time to wait after starting the mapkey, in milliseconds.
            Default is 0.

    Returns: None

    """
    data = {"script": script.replace("  ", ""), "delay": delay}
    return client._creoson_post("interface", "mapkey", data)


def plot(client, file_=None, dirname=None, driver=None):
    """Export a model plot.

    Args:
        client (obj):
            creopyson Client.
        `file_` (str, optional):
            Model name. Defaults is currently active model.
        dirname (str, optional):
            Destination directory.
            Defaults is Creo's current working directory.
        driver (str, optional):
            Driver export.
            Defaults is `POSTSCRIPT`.
            Valid values: POSTSCRIPT, JPEG, TIFF.

    Returns:
        dict:
            dirname (str): Directory of the output file
            filename (str): Name of the output file

    """
    data = {}
    if file_ is not None:
        data["file"] = file_
    else:
        active_file = client.file_get_active()
        if active_file:
            data["file"] = active_file["file"]
    if dirname is not None:
        data["dirname"] = dirname
    if driver is not None:
        data["driver"] = driver
    return client._creoson_post("interface", "plot", data)



"""Drawing module."""


def add_model(client, model, drawing=None):
    """Add a model to a drawing.

    Args:
        client (obj):
            creopyson Client.
        model (str):
            Model name.
        drawing (str, optional):
            Drawing name. Defaults is Current active drawing.

    Returns:
        None

    """
    data = {"model": model}
    if drawing is not None:
        data["drawing"] = drawing
    return client._creoson_post("drawing", "add_model", data)


def add_sheet(client, position=None, drawing=None):
    """Add a drawing sheet.

    Args:
        client (obj):
            creopyson Client.
        position (int, optional):
            Position to add the sheet.
            Defaults: Sheet will be added to the end.
        drawing (str, optional):
            Drawing name. Defaults is current active drawing.

    Returns:
        None

    """
    data = {}
    if position is not None:
        data["position"] = position
    if drawing is not None:
        data["drawing"] = drawing
    return client._creoson_post("drawing", "add_sheet", data)


def create(
    client,
    template,
    model=None,
    drawing=None,
    scale=None,
    display=None,
    activate=None,
    new_window=None,
):
    """Create a new drawing from a template.

    Args:
        client (obj):
            creopyson Client.
        template (str):
            Template
        model (str, optional):
            Model name. Defaults: Current active model.
        drawing (str, optional):
            New drawing name.
            Defaults: A name derived from the model's instance name.
        scale (float, optional):
            Drawing scale. Defaults is `1.0`.
        display (boolean, optional):
            Display the drawing after open. Defaults is False.
        activate (boolean, optional):
            Activate the drawing window after open. Defaults is False.
        new_window (boolean, optional):
            Open drawing in a new window. Defaults is False.

    Returns:
        (str): New drawing name.

    """
    data = {"template": template}
    if model is not None:
        data["model"] = model
    if drawing is not None:
        data["drawing"] = drawing
    if scale is not None:
        data["scale"] = scale
    if display is not None:
        data["display"] = display
    if activate is not None:
        data["activate"] = activate
    if new_window is not None:
        data["new_window"] = new_window
    return client._creoson_post("drawing", "create", data, "drawing")


def create_gen_view(
    client,
    model_view,
    point,
    drawing=None,
    view=None,
    sheet=None,
    model=None,
    scale=None,
    display_data=None,
    exploded=None,
):
    """Create general view on a drawing.

    Args:
        client (obj):
            creopyson Client
        model_view (str):
            Model view to use for the drawing view orientation.
        point (dict):
            Coordinates for the view in Drawing Units.
        drawing (str, optional):
            Drawing name. Defaults: current active drawing.
        view (str, optional):
            New view name. Defaults: the model_view parameter.
        sheet (int, optional):
            Sheet number. Defaults: current active sheet on the drawing.
        model (str, optional):
            Model for the view. Defaults: current active model on the drawing.
        scale (float, optional):
            View scale. Defaults: the sheet's scale.
        display_data (dict, optional):
            Display parameters used to create the view.
            Defaults: Creo defaults.
        exploded (boolean, optional):
            Whether to create the view as an exploded view. Defaults is False.

    Returns:
        None

    """
    data = {"model_view": model_view, "point": point}
    if drawing is not None:
        data["drawing"] = drawing
    if view is not None:
        data["view"] = view
    if sheet is not None:
        data["sheet"] = sheet
    if model is not None:
        data["model"] = model
    if scale is not None:
        data["scale"] = scale
    if display_data is not None:
        data["display_data"] = display_data
    if exploded is not None:
        data["exploded"] = exploded
    return client._creoson_post("drawing", "create_gen_view", data)
    # TODO: JLpoint Method for point
    # TODO: ViewDisplayData method for display_data


def create_proj_view(
    client,
    parent_view,
    point,
    drawing=None,
    view=None,
    sheet=None,
    display_data=None,
    exploded=None,
):
    """Create projection view on a drawing.

    When specifying the view coordinates, you should specify only an X or a Y
    coordinate to avoid confusion.  If you specify both coordinates, it
    appears Creo may be using whichever has the larger absolute value.

    Args:
        client (obj):
            creopyson Client
        parent_view (str):
            Parent view for the projection view.
        point (dict):
            Coordinates for the view, relative to the location
            of the parent view, in Drawing Units.
        drawing (str, optional):
            Drawing name. Defaults: current active drawing.
        view (str, optional):
            New view name. Defaults: Creo's default name for a new view.
        sheet (int, optional):
            Sheet number. Defaults: current active sheet on the drawing.
        display_data (dict, optional):
            Display parameters used to create the view.
            Defaults: the display parameters of the parent view.
        exploded (boolean, optional):
            Whether to create the view as an exploded view. Defaults is False.

    Returns:
        None

    """
    data = {"parent_view": parent_view, "point": point}
    if drawing is not None:
        data["drawing"] = drawing
    if view is not None:
        data["view"] = view
    if sheet is not None:
        data["sheet"] = sheet
    if display_data is not None:
        data["display_data"] = display_data
    if exploded is not None:
        data["exploded"] = exploded
    return client._creoson_post("drawing", "create_proj_view", data)


def create_symbol(
    client, symbol_file, point, drawing=None, replace_values=None, sheet=None
):
    """Add a symbol instance to a drawing.

    Args:
        client (obj):
            creopyson Client
        symbol_file (str):
            Name of the symbol file.
        point (dict):
            Coordinates for the symbol in Drawing Units.
        drawing (str, optional):
            Drawing name. Defaults: current active drawing.
        replace_values (dict, optional):
            Object containing replacement values for any
            variable text in the symbol. Defaults to None.
        sheet (int, optional):
            Sheet number (0 for all sheets).
            Defaults: the symbol will be added to all sheets.

    Returns:
        None

    """
    data = {"symbol_file": symbol_file, "point": point}
    if drawing is not None:
        data["drawing"] = drawing
    if replace_values is not None:
        data["replace_values"] = replace_values
    if sheet is not None:
        data["sheet"] = sheet
    return client._creoson_post("drawing", "create_symbol", data)


def delete_models(client, model=None, drawing=None, delete_views=None):
    """Delete one or more models from a drawing.

    Args:
        client (obj):
            creopyson Client
        model (str, optional):
            Model name (wildcard allowed: True).
            Defaults: all models will be deleted from the drawing.
        drawing (str, optional):
            Drawing name. Defaults: current active drawing.
        delete_views (boolean, optional):
            Whether to delete drawing views associated with the model.
            Defaults is False.

    Returns:
        None

    """
    data = {}
    if drawing is not None:
        data["drawing"] = drawing
    if model is not None:
        data["model"] = model
    if delete_views is not None:
        data["delete_views"] = delete_views
    return client._creoson_post("drawing", "delete_models", data)


def delete_sheet(client, sheet, drawing=None):
    """Delete a drawing sheet.

    An error will occur if you try to delete the only sheet in a drawing.

    Args:
        client (obj):
            creopyson Client
        sheet (int):
            Sheet number.
        drawing (str, optional):
            Drawing name. Defaults: current active drawing.

    Returns:
        None

    """
    data = {"sheet": sheet}
    if drawing is not None:
        data["drawing"] = drawing
    return client._creoson_post("drawing", "delete_sheet", data)


def delete_symbol_def(client, symbol_file, drawing=None):
    """Delete a symbol definition and its instances from a drawing.

    Args:
        client (obj):
            creopyson Client
        symbol_file (str):
            Name of the symbol file.
        drawing (str, optional):
            Drawing name. Defaults: current active drawing.

    Returns:
        None

    """
    data = {"symbol_file": symbol_file}
    if drawing is not None:
        data["drawing"] = drawing
    return client._creoson_post("drawing", "delete_symbol_def", data)


def delete_symbol_inst(client, symbol_id, drawing=None):
    """Delete a specific symbol instance from a drawing.

    Args:
        client (obj):
            creopyson Client
        symbol_id (str):
            ID of the symbol instance.
        drawing (str, optional):
            Drawing name. Defaults: current active drawing.

    Returns:
        None

    """
    data = {"symbol_id": symbol_id}
    if drawing is not None:
        data["drawing"] = drawing
    return client._creoson_post("drawing", "delete_symbol_inst", data)


def delete_view(client, view, drawing=None, sheet=None, del_children=None):
    """Delete a drawing view.

    Args:
        client (obj):
            creopyson Client
        view (str):
            View name.
        drawing (str, optional):
            Drawing name. Defaults: current active drawing.
        sheet (int, optional):
            Sheet number; if filled in, the view will only be deleted if it is
            on that sheet. Defaults: Delete the view from any sheet.
        del_children ([boolean, optional):
            Whether to also delete any children of the view. Defaults is False.

    Returns:
        None

    """
    data = {"view": view}
    if drawing is not None:
        data["drawing"] = drawing
    if sheet is not None:
        data["sheet"] = sheet
    if del_children is not None:
        data["del_children"] = del_children
    return client._creoson_post("drawing", "delete_view", data)


def get_cur_model(client, drawing=None):
    """Get the active model on a drawing.

    Args:
        client (obj):
            creopyson Client
        drawing (str, optional):
            Drawing name. Defaults: current active drawing.

    Returns:
        (str): Model name.

    """
    data = {}
    if drawing is not None:
        data["drawing"] = drawing
    return client._creoson_post("drawing", "get_cur_model", data, "file")


def get_cur_sheet(client, drawing=None):
    """Get the current drawing sheet.

    Args:
        client (obj):
            creopyson Client
        drawing (str, optional):
            Drawing name. Defaults: current active drawing.

    Returns:
        (int): Sheet number.

    """
    data = {}
    if drawing is not None:
        data["drawing"] = drawing
    return client._creoson_post("drawing", "get_cur_sheet", data, "sheet")


def get_num_sheets(client, drawing=None):
    """Get the number of sheets on a drawing.

    Args:
        client (obj):
            creopyson Client
        drawing (str, optional):
            Drawing name. Defaults: current active drawing.

    Returns:
        (int): Number of sheets.

    """
    data = {}
    if drawing is not None:
        data["drawing"] = drawing
    return client._creoson_post("drawing", "get_num_sheets", data, "num_sheets")


def get_sheet_format(client, sheet, drawing=None):
    """Get the drawing format file of drawing sheet.

    Args:
        client (obj):
            creopyson Client.
        sheet (int):
            Sheet number.
        drawing (str, optional):
            Drawing name. Defaults: current active drawing.

    Returns:
        (dict):
            file(str):
                Format file name, may be null if there is no current format.
            full_name(str):
                Format full name.
            common_name(str):
                Format common name.

    """
    data = {"sheet": sheet}
    if drawing is not None:
        data["drawing"] = drawing
    return client._creoson_post("drawing", "get_sheet_format", data)


def get_sheet_scale(client, sheet, drawing=None, model=None):
    """Get the scale of a drawing sheet.

    Args:
        client (obj):
            creopyson Client
        sheet (int):
            Sheet number.
        drawing (str, optional):
            Drawing name. Defaults: current active drawing.
        model (str, optional):
            Drawing model used to calculate the scale.
            Defaults: the active model on the drawing.

    Returns:
        (float): Sheet scale.

    """
    data = {"sheet": sheet}
    if drawing is not None:
        data["drawing"] = drawing
    if model is not None:
        data["model"] = model
    return client._creoson_post("drawing", "get_sheet_scale", data, "scale")


def get_sheet_size(client, sheet, drawing=None):
    """Get the size of a drawing sheet.

    Args:
        client (obj):
            creopyson Client
        sheet (int):
            Sheet number.
        drawing (str, optional):
            Drawing name. Defaults: current active drawing.

    Returns:
        (str): Sheet size.

    """
    data = {"sheet": sheet}
    if drawing is not None:
        data["drawing"] = drawing
    return client._creoson_post("drawing", "get_sheet_size", data, "size")


def get_view_loc(client, view, drawing=None):
    """Get the location of a drawing view.

    Args:
        client (obj):
            creopyson Client
        view (str):
            View name.
        drawing (str, optional):
            Drawing name. Defaults: current active drawing.

    Returns:
        (dict):
            x (float): X-coordinate of the view
            y (float): Y-coordinate of the view
            z (float): Z-coordinate of the view

    """
    data = {"view": view}
    if drawing is not None:
        data["drawing"] = drawing
    return client._creoson_post("drawing", "get_view_loc", data)
    # TODO: return a tuple (x,y,z)?


def get_view_scale(client, view, drawing=None):
    """Get the scale of a drawing view.

    Args:
        client (obj):
            creopyson Client
        view (str):
            View name.
        drawing (str, optional):
            Drawing name. Defaults: current active drawing.

    Raises:
        Warning: error message from creoson.

    Returns:
        (float): View scale.

    """
    data = {"view": view}
    if drawing is not None:
        data["drawing"] = drawing
    return client._creoson_post("drawing", "get_view_scale", data, "scale")


def get_view_sheet(client, view, drawing=None):
    """Get the sheet number that contains a drawing view.

    Args:
        client (obj):
            creopyson Client
        view (str):
            View name.
        drawing (str, optional):
            Drawing name. Defaults: current active drawing.

    Returns:
        (int): Sheet number.

    """
    data = {"view": view}
    if drawing is not None:
        data["drawing"] = drawing
    return client._creoson_post("drawing", "get_view_sheet", data, "sheet")


def is_symbol_def_loaded(client, symbol_file, drawing=None):
    """Check whether a symbol definition file is loaded into Creo.

    Args:
        client (obj):
            creopyson Client
        symbol_file (str):
            Name of the symbol file.
        drawing (str, optional):
            Drawing name. Defaults: current active drawing.

    Returns:
        (boolean): Whether the symbol definition is loaded into Creo.

    """
    data = {"symbol_file": symbol_file}
    if drawing is not None:
        data["drawing"] = drawing
    return client._creoson_post("drawing", "is_symbol_def_loaded", data, "loaded")


def list_models(client, model=None, drawing=None):
    """List the models contained in a drawing.

    Args:
        client (obj):
            creopyson Client
        model (str, optional):
            Model name filter (wildcards allowed: True).
            Defaults: no filter.
        drawing (str, optional):
            Drawing name. Defaults: current active drawing.

    Returns:
        (list:str): List of model names in the drawing.

    """
    data = {}
    if drawing is not None:
        data["drawing"] = drawing
    if model is not None:
        data["model"] = model
    return client._creoson_post("drawing", "list_models", data, "files")


def list_symbols(client, drawing=None, symbol_file=None, sheet=None):
    """List symbols contained on a drawing.

    Args:
        client (obj):
            creopyson Client
        drawing (str, optional):
            Drawing name. Defaults: current active drawing.
        symbol_file (str, optional):
            Symbol file name filter. Defaults: no filter.
        sheet (int, optional):
            Sheet number (0 for all sheets).
            Defaults: The symbol will be added to all sheets.

    Returns:
        (list:dict):
            List of symbols in the drawing.
                id (int): Symbol ID.
                symbol_name (str): Symbol name.
                sheet (int): Sheet number.

    """
    data = {}
    if drawing is not None:
        data["drawing"] = drawing
    if symbol_file is not None:
        data["symbol_file"] = symbol_file
    if sheet is not None:
        data["sheet"] = sheet
    return client._creoson_post("drawing", "list_symbols", data, "symbols")


def list_view_details(client, view=None, drawing=None):
    """List the views contained in a drawing, with more details.

    Args:
        client (obj):
            creopyson Client
        view (str, optional):
            View name filter (wildcards allowed: True). Defaults: no filter.
        drawing (str, optional):
            Drawing name. Defaults: current active drawing.

    Returns:
        (list:dict):
            List of views in the drawing
                name (str):
                    View name.
                sheet (int):
                    Sheet number.
                location (dict):
                    Coordonates
                        x (float): X-coordinate of the view
                        y (float): Y-coordinate of the view
                        z (float): Z-coordinate of the view
                text_height (float):
                    Text Heigh in Drawing Units.
                view_model (str):
                    View model name.
                simp_rep (str):
                    View simplified rep name.

    """
    data = {}
    if drawing is not None:
        data["drawing"] = drawing
    if view is not None:
        data["view"] = view
    return client._creoson_post("drawing", "list_view_details", data, "views")


def list_views(client, view=None, drawing=None):
    """List the views contained in a drawing.

    Args:
        client (obj):
            creopyson Client
        view (str, optional):
            View name filter (wildcards allowed: True). Defaults: no filter.
        drawing (str, optional):
            Drawing name. Defaults: current active drawing.

    Returns:
        (list:str): List of views in the drawing.

    """
    data = {}
    if drawing is not None:
        data["drawing"] = drawing
    if view is not None:
        data["view"] = view
    return client._creoson_post("drawing", "list_views", data, "views")


def load_symbol_def(client, symbol_file, symbol_dir=None, drawing=None):
    """Load a Creo symbol definition file into Creo from disk.

    Args:
        client (obj):
            creopyson Client
        symbol_file (str):
            Name of the symbol file.
        symbol_dir (str, optional):
            Directory containing the symbol file; if relative,
            assumed to be relative to Creo's current working directory.
            Defaults: Creo's current working directory.
        drawing (str, optional):
            Drawing name. Defaults: current active drawing.

    Returns:
        (dict): Symbol definition.
            id (int): ID of the loaded symbol.
            name (str): Symbol Name of the loaded symbol.

    """
    data = {"symbol_file": symbol_file}
    if drawing is not None:
        data["drawing"] = drawing
    if symbol_dir is not None:
        data["symbol_dir"] = symbol_dir
    return client._creoson_post("drawing", "load_symbol_def", data)


def regenerate(client, drawing=None):
    """Regenerate a drawing.

    Args:
        client (obj):
            creopyson Client
        drawing (str, optional):
            Drawing name. Defaults: current active drawing.

    Returns:
        None

    """
    data = {}
    if drawing is not None:
        data["drawing"] = drawing
    return client._creoson_post("drawing", "regenerate", data)


def regenerate_sheet(client, sheet=None, drawing=None):
    """Regenerate a sheet on a drawing.

    Args:
        client (obj):
            creopyson Client
        sheet (int, optional):
            Sheet number (0 for all sheets).
            Defaults: all sheets will be regenerated.
        drawing (str, optional):
            Drawing name. Defaults: current active drawing.

    Returns:
        None

    """
    data = {}
    if sheet is not None:
        data["sheet"] = sheet
    if drawing is not None:
        data["drawing"] = drawing
    return client._creoson_post("drawing", "regenerate_sheet", data)


def rename_view(client, view, new_view, drawing=None):
    """Rename a drawing view.

    Args:
        client (obj):
            creopyson Client
        view (str):
            Old view name.
        new_view (str):
            New view name.
        drawing (str, optional):
            Drawing name. Defaults: current active drawing.

    Returns:
        None

    """
    data = {"view": view, "new_view": new_view}
    if drawing is not None:
        data["drawing"] = drawing
    return client._creoson_post("drawing", "rename_view", data)


def scale_sheet(client, sheet, scale, drawing=None, model=None):
    """Set the scale of a drawing sheet.

    Args:
        client (obj):
            creopyson Client
        sheet (int):
            Sheet number.
        scale (float):
            View scale.
        drawing (str, optional):
            Drawing name. Defaults: current active drawing.
        model (str, optional):
            Drawing model to scale. Defaults: tThe active model on the drawing.

    Returns:
        None

    """
    data = {"sheet": sheet, "scale": scale}
    if drawing is not None:
        data["drawing"] = drawing
    if model is not None:
        data["model"] = model
    return client._creoson_post("drawing", "scale_sheet", data)


def scale_view(client, scale, view=None, drawing=None):
    """Set the scale of one or more drawing views.

    Args:
        client (obj):
            creopyson Client
        scale (float):
            View scale.
        view (str, optional):
            View name (wildcards allowed: True).
            Defaults: all views will be scaled.
        drawing (str, optional):
            Drawing name. Defaults: current active drawing.

    Returns:
        (dict)
            succes_views (list):
                List of view which were successfully scaled.
            failed_views (list):
                List of view which failed to scale.

    """
    data = {"scale": scale}
    if drawing is not None:
        data["drawing"] = drawing
    if view is not None:
        data["view"] = view
    return client._creoson_post("drawing", "scale_view", data)


def select_sheet(client, sheet, drawing=None):
    """Make a drawing sheet the current sheet.

    Args:
        client (obj):
            creopyson Client
        sheet (int):
            Sheet number.
        drawing (str, optional):
            Drawing name. Defaults: current active drawing.

    Returns:
        None

    """
    data = {"sheet": sheet}
    if drawing is not None:
        data["drawing"] = drawing
    return client._creoson_post("drawing", "select_sheet", data)


def set_cur_model(client, model, drawing=None):
    """Set the active model on a drawing.

    Args:
        client (obj):
            creopyson Client
        model (str):
            Model name.
        drawing (str, optional):
            Drawing name. Defaults: current active drawing.

    Returns:
        None

    """
    data = {"model": model}
    if drawing is not None:
        data["drawing"] = drawing
    return client._creoson_post("drawing", "set_cur_model", data)


def set_sheet_format(client, sheet, file_format, dirname=None, drawing=None):
    """Set the drawing format file of a drawing sheet.

    Args:
        client (obj):
            creopyson Client.
        sheet (int):
            Sheet number.
        file_format (str):
            Format file name.
        dirname (str, optional): Directory name containing the file format.
            Defaults to None is current working directory.
        drawing (str, optional):
            Drawing name. Defaults to None is current active drawing.

    Returns:
        None
    """
    data = {"sheet": sheet, "dirname": dirname}
    if drawing is not None:
        data["drawing"] = drawing
    if file_format is not None:
        data["file"] = file_format
    return client._creoson_post("drawing", "set_sheet_format", data)


def set_view_loc(client, view, point, drawing=None):
    """Set the location of a drawing view.

    Args:
        client (obj):
            creopyson Client
        view (str):
            View name.
        point (dict):
            Coordinates for the view in Drawing Units
        drawing (str, optional):
            Drawing name. Defaults: current active drawing.

    Returns:
        None

    """
    data = {"view": view, "point": point}
    if drawing is not None:
        data["drawing"] = drawing
    return client._creoson_post("drawing", "set_cur_model", data)


def view_bound_box(client, view, drawing=None):
    """Get the 2D bounding box for a drawing view.

    Args:
        client (obj):
            creopyson Client
        view (str):
            View name.
        drawing (str, optional):
            Drawing name. Defaults: current active drawing.

    Returns:
        (dict):
            xmin (float): Minimum X-coordinate of drawing view.
            xmax (float): Maximum X-coordinate of drawing view.
            ymin (float): Minimum Y-coordinate of drawing view.
            ymax (float): Maximum Y-coordinate of drawing view.

    """
    data = {"view": view}
    if drawing is not None:
        data["drawing"] = drawing
    return client._creoson_post("drawing", "view_bound_box", data)



"""Windchill testing."""

import creopyson
from .fixtures import mk_creoson_post_dict, mk_creoson_post_None


def test_windchill_authorize(mk_creoson_post_None):
    """Test windchill_authorize."""
    c = creopyson.Client()
    result = c.windchill_authorize("login", "password")
    assert result is None

def test_windchill_clear_workspace(monkeypatch, mk_creoson_post_None):
    """Test windchill_clear_workspace."""
    c = creopyson.Client()
    def fake_func():
        return "workspace"
    monkeypatch.setattr(c, 'windchill_get_workspace', fake_func)
    result = c.windchill_clear_workspace(workspace="MyWorkspace")
    assert result is None
    result = c.windchill_clear_workspace(filenames=["file1", "file2"])
    assert result is None


def test_windchill_create_workspace(mk_creoson_post_None):
    """Test windchill_create_workspace."""
    c = creopyson.Client()
    result = c.windchill_create_workspace("MyWorkspace", "Context")
    assert result is None


def test_windchill_delete_workspace(mk_creoson_post_None):
    """Test windchill_delete_workspace."""
    c = creopyson.Client()
    result = c.windchill_delete_workspace("MyWorkspace")
    assert result is None


def test_windchill_file_checked_out(monkeypatch, mk_creoson_post_dict):
    """Test file_checked_out."""
    c = creopyson.Client()
    def fake_func():
        return "workspace"
    monkeypatch.setattr(c, 'windchill_get_workspace', fake_func)
    result = c.windchill_file_checked_out("myfile.prt", workspace="workspace_name")
    assert result is True


def test_windchill_get_workspace(mk_creoson_post_dict):
    """Test windchill_authorize."""
    c = creopyson.Client()
    result = c.windchill_get_workspace()
    assert isinstance(result, (str))


def test_windchill_list_workspace_files(monkeypatch, mk_creoson_post_dict):
    """Test list_workspace_files."""
    c = creopyson.Client()

    def fake_func():
        return "workspace"
    monkeypatch.setattr(c, 'windchill_get_workspace', fake_func)
    result = c.windchill_list_workspace_files(filename="*.prt", workspace="workspace_name")
    assert isinstance(result, (list))


def test_windchill_list_workspaces(mk_creoson_post_dict):
    """Test list_workspaces."""
    c = creopyson.Client()
    result = c.windchill_list_workspaces()
    assert isinstance(result, (list))


def test_windchill_server_exists(mk_creoson_post_dict):
    """Test server_exists."""
    c = creopyson.Client()
    result = c.windchill_server_exists("server_path")
    assert result is True


def test_windchill_set_server(mk_creoson_post_None):
    """Test set_server."""
    c = creopyson.Client()
    result = c.windchill_set_server("server_path")
    assert result is None


def test_windchill_set_workspace(mk_creoson_post_None):
    """Test set_workspace."""
    c = creopyson.Client()
    result = c.windchill_set_workspace("workspace_name")
    assert result is None


def test_windchill_workspace_exists(mk_creoson_post_dict):
    """Test workspace_exists."""
    c = creopyson.Client()
    result = c.windchill_workspace_exists("server_path")
    assert result is True



"""Feature testing."""
import creopyson
import pytest
from .fixtures import (
    mk_creoson_post_dict,
    mk_creoson_post_None,
    mk_creoson_post_list,
    mk_getactivefile,
    mk_getactivefile,
)


def test_feature_delete(mk_creoson_post_None, mk_getactivefile):
    """Test delete."""
    c = creopyson.Client()
    result = c.feature_delete(
        file_="file", name="name", status="ACTIVE", type_="type", clip=True
    )
    assert result is None
    result = c.feature_delete(name=["name1", "name2"])
    assert result is None

    with pytest.raises(ValueError):
        c.feature_delete(status="FAKE")


def test_feature_delete_param(mk_creoson_post_None, mk_getactivefile):
    """Test delete_param."""
    c = creopyson.Client()
    result = c.feature_delete_param(file_="file", name="name", param="param")
    assert result is None
    result = c.feature_delete_param()
    assert result is None


def test_feature_list(mk_creoson_post_dict, mk_getactivefile):
    """Test list."""
    c = creopyson.Client()
    result = c.feature_list(
        file_="file",
        name="name",
        status="ACTIVE",
        type_="type",
        paths=True,
        no_datum=True,
        inc_unnamed=True,
        no_comp=True,
    )
    assert isinstance(result, (list))

    with pytest.raises(ValueError):
        c.feature_list(
            status="FAKE STATUS",
        )


def test_feature_list_params(mk_creoson_post_dict, mk_getactivefile):
    """Test list."""
    c = creopyson.Client()
    result = c.feature_list_params(
        file_="file",
        name="name",
        type_="type",
        no_datum=True,
        inc_unnamed=True,
        no_comp=True,
        param="param",
        value="param",
        encoded=True,
    )
    assert isinstance(result, (list))
    result = c.feature_list_params(
        param=["param", "other param"],
        name=12,
    )
    assert isinstance(result, (list))


def test_feature_list_group_features(mk_creoson_post_dict, mk_getactivefile):
    """Test list_group_features."""
    c = creopyson.Client()
    result = c.feature_list_group_features(
        "group_name",
        file_="file",
        type_="type",
    )
    assert isinstance(result, (list))
    result = c.feature_list_group_features("group_name")
    assert isinstance(result, (list))


def test_feature_list_pattern_features(mk_creoson_post_dict, mk_getactivefile):
    """Test list_group_features."""
    c = creopyson.Client()
    result = c.feature_list_pattern_features(
        "pattern_name",
        file_="file",
        type_="type",
    )
    assert isinstance(result, (list))
    result = c.feature_list_pattern_features("pattern_name")
    assert isinstance(result, (list))


def test_feature_list_selected(mk_creoson_post_dict, mk_getactivefile):
    """Test list_selected."""
    c = creopyson.Client()
    result = c.feature_list_selected()
    assert isinstance(result, (list))


def test_feature_param_exists(mk_creoson_post_dict, mk_getactivefile):
    """Test param_exists."""
    c = creopyson.Client()
    result = c.feature_param_exists(file_="file", name="name", param="param")
    assert result is True
    result = c.feature_param_exists(param=["param", "other param"])
    assert result is True


def test_feature_rename(mk_creoson_post_None, mk_getactivefile):
    """Test rename."""
    c = creopyson.Client()
    result = c.feature_rename("oldname", "new_name", file_="file")
    assert result is None
    c = creopyson.Client()
    result = c.feature_rename("oldname", "new_name")
    assert result is None
    result = c.feature_rename(1, "new_name", file_="file")
    with pytest.raises(TypeError):
        c.feature_rename(["list"], "new_name", file_="file")


def test_feature_resume(mk_creoson_post_None, mk_getactivefile):
    """Test resume."""
    c = creopyson.Client()
    result = c.feature_resume(
        file_="file", name="name", status="ACTIVE", type_="type", with_children=True
    )
    assert result is None
    result = c.feature_resume(
        name=["name", "other name"],
    )
    assert result is None
    result = c.feature_resume(
        name=123,
    )
    assert result is None
    with pytest.raises(ValueError):
        c.feature_resume(status="FAKE")


def test_feature_set_param(mk_creoson_post_None, mk_getactivefile):
    """Test set_param."""
    c = creopyson.Client()
    result = c.feature_set_param(
        "param",
        file_="file",
        name="name",
        type_="STRING",
        value=12,
        encoded=True,
        designate=True,
        description="This is a description",
        no_create=True,
    )
    assert result is None
    result = c.feature_set_param("param")
    assert result is None


def test_feature_suppress(mk_creoson_post_None, mk_getactivefile):
    """Test suppress."""
    c = creopyson.Client()
    result = c.feature_suppress(
        file_="file",
        name="name",
        status="ACTIVE",
        type_="type",
        clip=False,
        with_children=False,
    )
    assert result is None
    result = c.feature_suppress(
        name=["name", "other name"],
    )
    assert result is None
    result = c.feature_suppress(
        name=123,
    )
    assert result is None
    with pytest.raises(ValueError):
        c.feature_suppress(status="FAKE")


def test_feature_user_select_csys(mk_creoson_post_list, mk_getactivefile):
    """Test user_select_csys."""
    c = creopyson.Client()
    result = c.feature_user_select_csys(file_="file", max_=12)
    assert isinstance(result, (list))
    result = c.feature_user_select_csys()
    assert isinstance(result, (list))
